/*!
 * \file LU_matrix.c
 * \brief Functions for computing LU decomposition of any square matrix
 * 
 * \author Joan
 * \date 19/01/2014 (start)
 * \version 1.0
 */

/*!
 * \defgroup LU LU factorization
 * \brief Implement the LU decomposition of a square matrix with partial pivoting (rows or columns) or full pivoting are admitted
 * 
 * Let A be a square matrix. An LU factorization refers to the factorization of A, with proper row and/or column orderings or permutations, into two factors, a lower triangular matrix L with 1 in its diagonal and an upper triangular matrix U, that is A = LU.\n
 * 
 * It turns out that a proper permutation in rows (or columns) is sufficient for the LU factorization. The LU factorization with Partial Pivoting by rows refers often to the LU factorization with row permutations only, that can be denoted by PA = LU, where P is a permutation matrix which, when left-multiplied to A, reorders the rows of A.\n
 * 
 * The LU factorization with Partial Pivoting by columns refers to the LU factorization with column permutations, that can be denoted by AQ = LU where Q is a permutation matrix that reorders the columns of A.\n
 * 
 * It turns out that all square matrices can be factorized in this form, and the factorization is numerically stable in practice.\n
 * 
 * An LU factorization with full pivoting involves both row and column permutations, that can be denoted by PAQ = LU.
 *  
 * \section LUrectangular The LU factorization of a rectangular matrix
 * 
 * The LU factorization of a rectangular matrix \f$ A \in \mathbb{R}(m, n) \f$ can also be performed and it is guaranteed to exists if \f$ A[1:i][1:i] \f$ is not singular for \f$ i  = 1, \ldots, \min\{m, n\} \f$.\n
 * The calculation of a LU factorization of a rectangular matrix requires \f$ m n^2 - n^3 /3 \f$ flops and \a A can be overwritten by the strictly lower triangular portion of \f$ L \in \mathbb{R}(m, \min\{m, n\}) \f$ and the upper triangular portio of \f$ U \in \mathbb{R} (\min\{m, n\}, n)\f$.
 * 
 * @{
 */

#include "LU_matrix.h"

/*!
 * \def _OPENMP_LU_DIMENSION
 * \brief Minimum value for using the openmp library in LU decomposition
 * \showinitializer
 */
#define _OPENMP_LU_DIMENSION 1e2

/*!
 * \fn double max_row(size_t m, double ** const A, size_t idx, size_t * const row)
 * \brief Find the maximum value of a column based on the same row index
 * 
 * \param[in] m Dimension of matrixs
 * \param[in] A Matrix m-by-n where will be read the column
 * \param[in] idx Column index
 * \param[out] row Row index with the first maximum value
 * 
 * \return The first maximum value in absolute value in the column index
 * 
 * \pre Dimension is not 0, the indexs is less than the dimension and the pointer is not NULL
 * \note The row indexes can be the same
 */
double max_row(size_t m, double ** const A, size_t idx, size_t * const row) {
  typeof(m) i;
  typeof(**A) max, aux;
  
  *row = idx;
  max = fabs(A[idx][idx]);
  
  for (i = idx + 1; i < m; i ++) {
    aux = fabs(A[i][idx]);
    if (max < aux) {
      max = aux;
      *row = i;
    }
  }
  
  return max;
}

/*!
 * \fn double max_col(size_t n, double ** const A, size_t idx, size_t * const col)
 * \brief Find the first maximum value of a column based on the same column index
 * 
 * \param[in] n Dimension of matrixs
 * \param[in] A Matrix m-by-n where will be read the column
 * \param[in] idx Row index
 * \param[out] col Column index with the first maximum value
 * 
 * \return The first maximum value in absolute value in the row index
 * 
 * \pre The indexs is less than the dimension and the pointer is not NULL
 * \note The column indexes can be the same
 */
double max_col(size_t n, double ** const A, size_t idx, size_t * const col) {
  typeof(n) i;
  typeof(**A) max, aux;
  
  *col = idx;
  max = fabs(A[idx][idx]);
  
  for (i = idx + 1; i < n; i ++) {
    aux = fabs(A[idx][i]);
    if (max < aux) {
      max = aux;
      *col = i;
    }
  }
  
  return max;
}

/*!
 * \fn double max_row_col(size_t m, size_t n, double ** const A, size_t idx, size_t * const row, size_t * const col)
 * \brief Find the maximum value of a column and row based on the same column and row index
 * 
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in] A Matrix m-by-n where will be read the column
 * \param[in] idx Index
 * \param[out] row Row index
 * \param[out] col Column index
 * 
 * \return The maximum value in absolute value
 * 
 * \pre 
 *      - The indexs is less than the dimension
 *      - The pointers are not NULL
 * \post Either row index or else column index is different than \a idx index
 * \note The column indexes can be the same
 */
double max_row_col(size_t m, size_t n, double  ** const A, size_t idx, size_t * const row, size_t * const col) {
  typeof(n) i;
  typeof(**A) max, aux;
  
  *row = *col = idx;
  max = fabs(A[idx][idx]);
  
  for (i = idx + 1; i < MIN(m, n); i ++) {
    aux = fabs(A[i][idx]);
    if (max < aux) {
      max = aux;
      *row = i; *col = idx;
    }
    aux = fabs(A[idx][i]);
    if (max < aux) {
      max = aux;
      *col = i; *row = idx;
    }
  }
  
  if (m < n) {
    for (; i < n; i ++) {
      aux = fabs(A[idx][i]);
      if (max < aux) {
        max = aux;
        *col = i;
      }
    }
  } else if (n < m) {
    for (; i < m; i ++) {
      aux = fabs(A[i][idx]);
      if (max < aux) {
        max = aux;
        *row = i;
      }
    }
  }
    
  return max;
}

/*!
 * \fn void swap_bit(size_t i, size_t j, size_t * const x, size_t * const y)
 * \brief Swap two values from vector x to y
 * 
 * \param[in] i Index
 * \param[in] j Index
 * \param[in] x Source vector
 * \param[out] y Destination vector
 * 
 * \note The vectors can be the same. If one of them is NULL, anything is done
 */
void swap_bit(size_t i, size_t j, size_t * const x, size_t * const y) {
  if (x == NULL || y == NULL) return;
  
  y[i] = x[i] ^ x[j];
  y[j] = y[i] ^ x[j];
  y[i] = y[i] ^ y[j];
}

/*!
 * \fn void swap_row(size_t row1, size_t row2, size_t n, double ** const A, double ** const B)
 * \brief Swap two rows from matrix A to B based on the same minimum row index
 * 
 * \param[in] row1 Row index
 * \param[in] row2 Row index
 * \param[in] n Columns
 * \param[in] A Matrix m-by-n where will be read the rows
 * \param[out] B Matrix m-by-n where will be saved the rows
 * 
 * \pre 
 *      - The indexs are less than the number of rows
 *      - The pointers are not NULL
 * \post The swapping is in B
 * \note 
 *      - The row indexes can be the same
 *      - The matrix can be the same
 */
void swap_row(size_t row1, size_t row2, size_t n, double ** const A, double ** const B) {
  typeof(n) i;
  typeof(*A) tmp;
  
  if (A == B) {    
    tmp = A[row1];
    A[row1] = A[row2];
    A[row2] = tmp;
  } else {    
    #pragma omp parallel if (n > _OPENMP_LU_DIMENSION)
    {
    #pragma omp for private(i)
    for (i = 0; i < n; i ++) {
      B[row1][i] = A[row2][i];
      B[row2][i] = A[row1][i];
    }
    }
  }
}

/*!
 * \fn void swap_col(size_t col1, size_t col2, size_t m, double ** const A, double ** const B)
 * \brief Swap two columns from matrix A to B based on the same minimum column index
 * 
 * \param[in] col1 Col index
 * \param[in] col2 Col index
 * \param[in] m Rows
 * \param[in] A Matrix m-by-n where will be read the rows
 * \param[out] B Matrix m-by-n where will be saved the rows
 * 
 * \pre 
 *      - The indexs are less than the number of rows
 *      - The pointers are not NULL
 * \post The swapping is in B
 * \note 
 *      - The column indexes can be the same
 *      - The matrix can be the same
 */
void swap_col(size_t col1, size_t col2, size_t m, double ** const A, double ** const B) {
  typeof(m) i;
  typeof(**A) tmp;
  
  #pragma omp parallel if (m > _OPENMP_LU_DIMENSION) private(tmp)
  {
  #pragma omp for private(i) nowait
  for (i = 0; i < m; i ++) {
    tmp = A[i][col1];
    B[i][col1] = A[i][col2];
    B[i][col2] = tmp;
  }
  }
}

/*!
 * \fn void * lu_flag(size_t m, size_t n, double ** const A, double ** const LU, size_t * const pr, size_t * const pc, double tol, unsigned int flag)
 * \brief Compute the LU factorization of a matrix m-by-n with a specific pivoting
 * 
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in] A Matrix m-by-n where will be computed LU factorization
 * \param[out] LU Matrix m-by-n where has been computed LU factorization
 * \param[out] pr Vector m-dimensional where will be saved the pivoting by rows
 * \param[out] pc Vector n-dimensional where will be saved the pivoting by columns
 * \param[in] tol Tolerance that indicates a diagonal's element is valid for applying the Gaussian elimination
 * \param[in] flag Type of pivoting
 * 
 * \retval NULL If tolerance is greater than the pivot in absolute value
 * \retval 1 Otherwise
 * 
 * \pre 
 *      - The dimension are greater than 1 
 *      - The pointers \a A and \a LU are not NULL 
 *      - The tolerance is positive
 *      - Row vector permutation pointer and Column vector permutation are either NULL or else different
 * \post 
 *      - The LU factorization has been saved in \a LU
 *      - The vector permutations (\a pr and \a pc) can have been initialized, so the free should be done
 * \note 
 *      - LU and A can be the same 
 *      - The \a flags can be #WHITOUT_PIVOTING , #PARTIAL_PIVOTING_ROW , #PARTIAL_PIVOTING_COL or #FULL_PIVOTING
 */
void * lu_flag(size_t m, size_t n, double ** const A, double ** const LU, size_t * const pr, size_t * const pc, double tol, unsigned int flag) {
  assert(m > 1 && n > 1 && tol >= 0.0);
  
  typeof(n) i, j, k;
  
  if (A != LU) copy_matrix(m, n, A, LU);
  
  switch (flag) {
    case WHITOUT_PIVOTING:
      for (i = 0; i < MIN(m, n); i ++) {      
        if (fabs(LU[i][i]) <= tol) return NULL;
        
        #pragma omp parallel if ((m-i) > _OPENMP_LU_DIMENSION) default(shared) private(j, k)
        {
        #pragma omp for
        for (j = i + 1; j < m; j ++) {
          if (LU[j][i] == 0.0) continue;
    
          LU[j][i] /= LU[i][i];
          for (k = i + 1; k < n; k ++) LU[j][k] -= LU[j][i] * LU[i][k];
        }
        }
      }
      break;
    case PARTIAL_PIVOTING_ROW:
      permutation_init(m, pr);
      for (i = 0; i < MIN(m, n); i ++) {
        if (max_row(m, LU, i, &j) <= tol) return NULL;
        
        if (j != i) {
          swap_row(i, j, n, LU, LU);
          swap_bit(i, j, pr, pr);
        }
        
        #pragma omp parallel if ((m-i) > _OPENMP_LU_DIMENSION) default(shared) private(j, k)
        {
        #pragma omp for
        for (j = i + 1; j < m; j ++) {
          if (LU[j][i] == 0.0) continue;
    
          LU[j][i] /= LU[i][i];
          for (k = i + 1; k < n; k ++) LU[j][k] -= LU[j][i] * LU[i][k];
        }
        }
      }
      break;
    case PARTIAL_PIVOTING_COL:
      permutation_init(n, pc);
      for (i = 0; i < n; i ++) {        
        if (max_col(n, LU, i, &j) <= tol) return NULL;
        
        if (j != i) {
          swap_col(i, j, m, LU, LU);
          swap_bit(i, j, pc, pc);
        }
        
        #pragma omp parallel if ((m-i) > _OPENMP_LU_DIMENSION) default(shared) private(j, k)
        {
        #pragma omp for
        for (j = i + 1; j < m; j ++) {
          if (LU[j][i] == 0.0) continue;
    
          LU[j][i] /= LU[i][i];
          for (k = i + 1; k < n; k ++) LU[j][k] -= LU[j][i] * LU[i][k];
        }
        }
      }
      break;
    case FULL_PIVOTING:
      permutation_init(m, pr);
      permutation_init(n, pc);
      for (i = 0; i < n; i ++) {
        if (max_row_col(m, n, LU, i, &j, &k) <= tol) return NULL;
        
        if (j != i) {
          swap_row(i, j, n, LU, LU);
          swap_bit(i, j, pr, pr);
        }
        else if (k != i) {
          swap_col(i, k, m, LU, LU);
          swap_bit(i, j, pc, pc);
        }
        
        #pragma omp parallel if ((m-i) > _OPENMP_LU_DIMENSION) default(shared) private(j, k)
        {
        #pragma omp for
        for (j = i + 1; j < m; j ++) {
          if (LU[j][i] == 0.0) continue;
      
          LU[j][i] /= LU[i][i];
          for (k = i + 1; k < n; k ++) LU[j][k] -= LU[j][i] * LU[i][k];
        }
        }
      }
      break;
    default:
      perror("The flag for LU decomposition is not valid");
      break;
  }
  
  /* Without consider LU argument */
//   for (i = 0; i < n; i ++) {
//     j = max_row(n, A, i);
//     
//     if (fabs(A[j][j]) <= tol)
//       return NULL;
//     
//     if (j != i) {
//       swap_row(i, j, n, A, A);
//       p[i] = j;
//     } else p[i] = i;
//     
//     for (j = i + 1; j < n; j ++) {
//       A[j][i] = A[j][i] / A[i][i];
//       
//       for (k = i + 1; k < n; k ++)
//         A[j][k] -= A[j][i] * A[i][k];
//     }
//   }
  
  return (void *) 1;
}

/*!
 * \fn void * lu(size_t m, size_t n, double ** const A, double ** const LU, size_t * const pr, size_t * const pc, double tol)
 * \brief Compute the LU factorization of a matrix n-by-n
 * 
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in] A Matrix n-by-n where will be computed LU factorization
 * \param[out] LU Matrix n-by-n where has been computed LU factorization
 * \param[out] pr Vector n-dimensional where will be saved the pivoting by rows
 * \param[out] pc Vector n-dimensional where will be saved the pivoting by columns
 * \param[in] tol Tolerance that indicates a diagonal's element is valid for applying the Gaussian elimination
 * 
 * \retval NULL If tolerance is greater than the pivot in absolute value
 * \retval 1 Otherwise
 * 
 * \pre 
 *      - The dimension is greater than 1 
 *      - The pointers \a A and \a LU are not NULL 
 *      - The tolerance is positive
 * \post 
 *      - The LU factorization has been saved in \a LU
 *      - The vector permutations (\a pr and \a pc) have been initialized, so the free should be done
 * \note 
 *      - LU and A can be the same 
 *      - If \a pr is NULL and \a pc is NULL, pivoting will not be done
 *      - If \a pr is NULL and \a pc is not NULL, partial pivoting by columns will be done
 *      - If \a pr is not NULL and \a pc is NULL, partial pivoting by rows will be done
 *      - If \a pr is not NULL and \a pc is not NULL, full pivoting will be done
 */
void * lu(size_t m, size_t n, double ** const A, double ** const LU, size_t * const pr, size_t * const pc, double tol) {
  assert(n > 1 && tol >= 0.0);
  
  if (pr == NULL && pc == NULL)
    return lu_flag(m, n, A, LU, pr, pc, tol, WHITOUT_PIVOTING);
  
  else if (pc == NULL)
    return lu_flag(m, n, A, LU, pr, pc, tol, PARTIAL_PIVOTING_ROW);
  
  else if (pr == NULL)
    return lu_flag(m, n, A, LU, pr, pc, tol, PARTIAL_PIVOTING_COL);
  
  else
    return lu_flag(m, n, A, LU, pr, pc, tol, FULL_PIVOTING);
}

/*!
 * @}
 */