/*!
 * \file QR_matrix.c
 * \brief Functions for computing QR decomposition of any matrix
 * 
 * \author Joan
 * \date 10/12/2013 (start)
 * \version 1.0
 */

/*!
 * \defgroup QR QR factorization
 * \brief ..
 * 
 * @{
 */

#include "QR_matrix.h"

/*!
 * \def _OPENMP_QR_DIMENSION
 * \brief Minimum value for using the openmp library in QR decomposition
 * \showinitializer
 */
#define _OPENMP_QR_DIMENSION 1e2

/*!
 * \fn void premult_Q_vector(size_t m, size_t n, double ** const QR, double * const b)
 * \brief Compute \f$P_1\cdots P_nb\f$ where \f$b\f$ and \f$P_i\f$ are respectively m-by-1 and m-by-m matrixs
 * 
 * \param[in] m Rows of linear system's matrix
 * \param[in] n Columns of linear system's matrix
 * \param[in] QR Householder vector of dimension n
 * \param[in,out] b Vector m-dimensional where will be saved the result
 * 
 * \pre It has been computed the QR descomposition over the matrix input
 * \post The calculation was carried out
 */
void premult_Q_vector(size_t m, size_t n, double ** const QR, double * const b) {
  assert(m > 1 && n > 1);  
  size_t i, j;
  double aux, tmp;
    
  for (i = 0; i < n - (m == n); i ++) {
    aux = 1.0;
    tmp = b[i];
    for (j = i + 1; j < m; j ++) {
      aux += QR[j][i] * QR[j][i];
      tmp += QR[j][i] * b[j];
    }
    
    tmp *= -2 / aux;
    
    b[i] += tmp;
    for (j = i + 1; j < m; j ++)
      b[j] += tmp * QR[j][i];
  }
}

/*!
 * \fn void * qr(size_t m, size_t n, double ** const A, double ** const QR)
 * \brief Compute the QR factorization of a matrix m-by-n
 * 
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in] A Matrix m-by-n where will be computed QR factorization
 * \param[out] QR Matrix m-by-n where has been computed QR factorization
 * 
 * \retval NULL If the memory can not be allocated
 * \retval 0 If some error ocurred and number of columns where has been applied the decomposition in QR
 * \return Otherwise the last index for the column changed
 * 
 * \note QR and A can be the same
 */
void * qr(size_t m, size_t n, double ** const A, double ** const QR) {
  assert(m >= n && n > 1);
  
  double *v, *w;  
  if ((v = (double *) malloc(m*sizeof(double))) == NULL) {
    perror("Allocated memory for v");
    return NULL;
  }
  
  if ((w = (double *) malloc(n*sizeof(double))) == NULL) {
    free(v);
    perror("Allocated memory for w");
    return NULL;
  }
  
  size_t i, j, ret;
  j = 0;
  if (householder_matrix(m, A, v, j) == NULL || premult_matrix(m, n, A, QR, v, w, j) == NULL) {
    free(w); free(v);
    return (void *) 0;
  }
  
  for (i = 1; i < m; i ++)
    QR[i][j] = v[i];
  
  for (j = 1; j < n - (n == m); j ++) {
    if (householder_matrix(m, QR, v, j) == NULL || premult_matrix(m, n, QR, QR, v, w, j) == NULL) {
      j = 1;
      break;
    }
    
    if (j < m)
      for (i = j + 1; i < m; i ++)
        QR[i][j] = v[i];
  }
  
  ret = j - 1;
      
  /* Free memory */
  free(w);
  free(v);
  return (void *) ret;
}
/*!
 * @}
 */