/*!
 * \file householder.c
 * \brief Functions for using householder vectors and matrix
 * 
 * \author Joan
 * \date 21/01/2014 (start)
 * \version 1.0
 */

/*!
 * \defgroup householder Householder
 * \brief Implements the householder matrix, the householder vector and some operations with them like as premultiply a householder matrix with other matrix.
 * 
 * The householder matrix with householder vector \f$ v \f$ is
 * \f[ P_v = I - \frac{2}{v^t v} vv^t \f]
 * 
 * And the householder vector of a vector \f$ x \ne 0 \f$ is \f$ v = x + sign(x_1)||x||_2 \f$.
 * @{
 */

#include "householder.h"

/*!
 * \def _OPENMP_HOUSEHOLDER_DIMENSION
 * \brief Minimum value for using openmp in \ref householder
 * \showinitializer
 */
#define _OPENMP_HOUSEHOLDER_DIMENSION 1e2

/*!
 * \fn void * householder_vector(size_t n, double * const x, double * const v)
 * \brief Derive the householder vector
 * 
 * \param[in] n Dimension of vector
 * \param[in] x Input vector
 * \param[out] v Householder vector
 * 
 * \retval NULL If the vector's elements of x are all 0
 * \retval 1 Otherwise
 * 
 * \pre The dimension is not zero
 * 
 * \note The vector pointers x and v can be the same
 */
void * householder_vector(size_t n, double * const x, double * const v) {
  assert(n > 0);
  
  typeof(*x) aux;
  typeof(n) i;
  
  /* Euclidian norm */  
  aux = euclidian_norm(n, x);
  
  if (aux == 0) return NULL;
      
  /* Essential part of Householder vector */
  aux = *x + ((*x >= 0) - (*x < 0))*aux;
  
  *v = 1;
  
  #pragma omp parallel if(n > _OPENMP_HOUSEHOLDER_DIMENSION) default(shared)
  {
  #pragma omp for private(i) nowait
  for (i = 1; i < n; i ++)
    v[i] = x[i] / aux;
  } /*-- End of parallel region --*/
  
  return (void *) 1;
}

/*!
 * \fn void * householder_matrix(size_t m, double ** const A, double * const v, size_t idx)
 * \brief Derive the householder vector using a matrix
 * 
 * \param[in] m Dimension of vector
 * \param[in] A Matrix m-by-n
 * \param[in] idx Index of matrix where is started to compute the vector householder
 * \param[out] v Householder vector
 * 
 * \retval NULL If the vector's elements of v[idx:] are all 0
 * \retval 1 Otherwise
 * 
 * \pre The dimension is > 0 and idx is < than number of columns of A
 * \post v[idx:] has been modified
 */
void * householder_matrix(size_t m, double ** const A, double * const v, size_t idx) {
  assert(0 < m && idx < m);
  
  typeof(**A) aux;
  typeof(m) i;
  
  /* Euclidian norm */
  #pragma omp parallel if((m-idx) > _OPENMP_HOUSEHOLDER_DIMENSION) default(shared)
  {
  #pragma omp for private(i) reduction(+:aux) nowait
  for (i = idx; i < m; i ++)
    aux += A[i][idx] * A[i][idx];       /* Inner product */
  } /*-- End of parallel region --*/
  
  if (aux == 0) return NULL;
  
  aux = sqrt(aux);                      /* Euclidian norm */
  
  #ifdef DEBUG_QR_PRINT
    printf("DEBUG_QR householder_matrix: Euclidian norm of x = %e\n", aux);
  #endif
  
  /* Essential part of Householder vector */
  aux = A[idx][idx] + ((A[idx][idx] >= 0) - (A[idx][idx] < 0))*aux;
  
  #ifdef DEBUG_QR_PRINT
    printf("DEBUG_QR householder_matrix: beta = %e\n", aux);
  #endif
  
  v[idx] = 1;
  
  #pragma omp parallel if((m-idx-1) > _OPENMP_HOUSEHOLDER_DIMENSION) default(shared)
  {
  #pragma omp for private(i) nowait
  for (i = idx + 1; i < m; i ++)
    v[i] = A[i][idx] / aux;
  } /*-- End of parallel region --*/
  
  #ifdef DEBUG_QR_PRINT
    printf("DEBUG_QR householder_matrix: solution v\n");
    print_vector(idx, m, v, "v");
  #endif
    
  return (void *) 1;
}

/*!
 * \fn void premult_matrix(size_t m, size_t n, double ** const A, double ** const QR, double * const v, double * const w, size_t idx)
 * \brief Compute \f$P_vA\f$ where \f$A\f$ and \f$P_v\f$ are respectively m-by-n and n-by-n matrixs
 * 
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in] A Matrix m-by-n
 * \param[in] v Householder vector of dimension m
 * \param[in] w Auxiliar vector of dimension n
 * \param[in] idx Index of vectors where is started to compute
 * \param[out] QR Matrix m-by-n where will be saved the result
 * 
 * \pre n <= m and idx < m
 */
void * premult_matrix(size_t m, size_t n, double ** const A, double ** const QR, double * const v, double * const w, size_t idx) {
  assert(n > 0 && n <= m && idx < m);
  
  typeof(**A) aux;
  typeof(n) i, j;
  
  /* Compute the inner product */  
  #pragma omp parallel if((MAX(m,n)-idx) > _OPENMP_HOUSEHOLDER_DIMENSION) default(shared) private(i)
  {
  #pragma omp for reduction(+:aux)
  for (i = idx + 1; i < m; i ++)
    aux += v[i] * v[i];
  
  #pragma omp single
  {
  #ifdef DEBUG_QR_PRINT
    printf("DEBUG_QR premult: inner_product = %e\n", aux);
  #endif
    
  /* A Householder vector is not 0, by definition */  
  aux = -2.0 / (aux + 1.0);

  #ifdef DEBUG_QR_PRINT
    printf("DEBUG_QR premult: beta = %f, j = %zu, m = %zu, n = %zu\n", aux, idx, m, n);
  #endif
  }
  
  /* Columns optimization */
//   typeof(**A) tmp;
//   #pragma omp for private(tmp, j)
//   for (i = idx; i < n; i ++) {
//     tmp = 0.0;
//     
//     for (j = idx; j < m; j ++)
//       tmp +=  A[j][i] * v[j]; /* Here is used the assumption n <= m */
//     
//     w[i] = aux * tmp;
//     
//     /* The row idx must be changed 
//      * but the column idx only the diagonal value 
//      */
//     QR[idx][i] = A[idx][i] + v[idx] * w[i];
//   }

  /* Rows optimization */
  #pragma omp for
  for (i = idx; i < n; i ++)
    w[i] = 0.0;
  
  #pragma omp single
  {
  for (j = idx; j < m - 1; j ++)
    for (i = idx; i < n; i ++)
      w[i] += A[j][i] * v[j];
  }
    
  #pragma omp for
  for (i = idx; i < n; i ++) {
    w[i] = aux * (w[i] + A[j][i] * v[j]);
    
    /* The row idx must be changed 
     * but the column idx only the diagonal value 
     */
    QR[idx][i] = A[idx][i] + v[idx] * w[i];
  }
  
  /* Post increment. 
   * Hence in the below loops not sum in initial counter 
   */
  #pragma omp single
  {
  idx ++; 
  }
  
  /* Compute the other elements */
  #pragma omp for private(j) nowait
  for (i = idx; i < m; i ++)
    for (j = idx; j < n; j ++)
      QR[i][j] = A[i][j] + v[i] * w[j];
  } /*-- End of parallel region --*/
  
  return (void *) 1;
}

/*!
 * \fn void premult_vector(size_t n, double * const v, double * const b)
 * \brief Compute \f$P_vb\f$ where \f$b\f$ and \f$P_v\f$ are respectively n-by-1 and n-by-n matrixs
 * 
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in] v Householder vector of dimension n
 * \param[in,out] b Vector n-dimensional where will be saved the result
 */
void premult_vector(size_t n, double * const v, double * const b) {
  assert(n > 0);
  typeof(n) i;
  typeof(*v) aux, tmp;
  
  #pragma omp parallel if((n-1) > _OPENMP_HOUSEHOLDER_DIMENSION) default(shared) private(i)
  {
  #pragma omp for reduction(+:aux,tmp)
  for (i = 1; i < n; i ++) {
    aux += v[i] * v[i];
    tmp += v[i] * b[i];
  }
  
  #pragma omp single
  {
  tmp = -2 * (tmp + b[0]) / (aux + 1);  
  b[0] += tmp;
  }
  
  #pragma omp for nowait
  for (i = 1; i < n; i ++)
    b[i] +=  tmp * v[i];
  } /*-- End of parallel region --*/
}

/*!
 * \fn void postmult(size_t m, size_t n, double **A, double * const v, double * const w)
 * \brief Compute \f$AP_v\f$ where \f$A\f$ and \f$P_v\f$ are respectively m-by-n and n-by-n matrixs
 * 
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in,out] A Matrix m-by-n where will be saved the result
 * \param[in] v Householder vector of dimension n
 * \param[in] w Auxiliar vector of dimension m
 */
void postmult(size_t m, size_t n, double ** const A, double * const v, double * const w) {
  assert(n > 0);
  typeof(**A) aux, tmp;
  
  aux = inner_product(m, v);
  assert(aux > 0);
  aux = -2 / aux;
  
  #ifdef DEBUG_QR_PRINT
  printf("DEBUG_QR premult: beta = %e\n", aux);
  #endif
  
  size_t i, j;  
  for (i = 0; i < m; i ++) {
    tmp = 0.0;
    
    for (j = 0; j < n; j ++)
      tmp += A[i][j] * v[j];
    
    w[i] = aux * tmp;
  }
  
  for (i = 0; i < m; i ++)
    for (j = 0; j < n; j ++)
      A[i][j] += w[i] * v[j];
}
/*!
 * @}
 */