/*! 
 * \file pic.h
 * \brief Header of periodic invariant curve
 *
 * \author Joan Gimeno
 * \date 22/01/2014 (start)
 * \version 1.0
 */

#ifndef PIC_H
#define PIC_H

#ifdef DEBUG
#define _PIC_DEBUG
#endif

#include "newton.h"

/*!
 * \typedef pic
 * \struct pic 
 * \brief Periodic Invariant Curve
 * 
 * Fields for a periodic invariant curve
 * 
 * \var pic::n
 * \brief Dimension of the curve's points
 * \pre Is not zero
 * 
 * \var pic::N
 * \brief Mesh size of one-dimensional torus
 * \pre Is not zero
 * 
 * \var pic::x
 * \brief (2N+1)n points that mesh the periodic invariant curve
 * 
 * \var pic::X
 * \brief n(2N + 1) Fourier coefficients
 * 
 * \var pic::DF
 * \brief Pointer to (2N+1)n-by-(2N+1)n matrix for Newton methods
 * 
 * \var pic::w
 * \brief Irrational number
 * 
 * \var pic::autonomous
 * \brief Flag that indicates if the dynamic system is autonomous or not \ref PIC_AUTONOMOUS and \ref PIC_NAUTONOMOUS
 * 
 * \var pic::f
 * \brief Pointer to function that is given by user
 * 
 * \var pic::solve
 * \brief Pointer to function that is given by user for solving a linear of equation system
 */
typedef struct pic {
  size_t n;
  size_t N;
  double *x;
  double *X;
  double **DF;
  double w;
  short unsigned int autonomous;
  void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double);
  void * (*solve)(size_t, size_t, double ** const, double * const, double * const, double);
} pic;

/*!
 * \def PIC_AUTONOMOUS
 * \brief The dynamic system is autonomous
 * \hideinitializer
 */
#define PIC_AUTONOMOUS 1
/*!
 * \def PIC_NAUTONOMOUS
 * \brief The dynamic system is not autonomous
 * \hideinitializer
 */
#define PIC_NAUTONOMOUS 0

/***********************
 * Function prototypes
 ***********************/

/* Get and Set global variables */
unsigned int pic_get_max_iterations();
unsigned int pic_set_max_iterations(unsigned int max_iter);

size_t pic_get_n(struct pic * const);
size_t pic_get_N(struct pic * const);
double * pic_get_x(struct pic * const);
double * pic_get_X(struct pic * const);
double ** pic_get_DF(struct pic * const);
double pic_get_w(struct pic * const);

/* Get fields of a pic */
size_t pic_get_mesh_size(struct pic * const);
size_t pic_get_dimension_size(struct pic * const);

/* Set fields of a pic */
int pic_set_mesh_size(struct pic * const p, size_t N);
int pic_set_dimension_size(struct pic * const p, size_t n);

int pic_set_function_pointer(struct pic * const p, _FUNCTION_PROTOTYPE);

int pic_set_function_solve(struct pic * const p, _SOLVE_PROTOTYPE);

/* Create and initialize values */
struct pic * pic_create(size_t n, size_t N, double w, double * const X, _FUNCTION_PROTOTYPE, _SOLVE_PROTOTYPE);
// struct pic * pic_create(size_t n, size_t N, double w, double * const X, short unsigned int autonomous);

/* Program execute */
void * pic_execute(struct pic * const p, double tol);

/* Differential of the process */
double ** pic_differential(struct pic * const p, size_t * const m, size_t * const n);

/* Load a Fourier serie */
double * pic_load_fourier_coef(size_t n, size_t N, FILE * const f);
double * pic_loadas_fourier_coef(size_t n, size_t N, char * const name);

/* Save the results */
int pic_save_fourier_coef(struct pic * const p, FILE * const f);
int pic_saveas_fourier_coef(struct pic * const p, char * const name);

int pic_save_curve(struct pic * const p, FILE * const f);
int pic_saveas_curve(struct pic * const p, char * const name);

int pic_saveas_differential(struct pic * const p, char * const name);
int pic_save_differential(struct pic * const p, FILE * const f);

int pic_saveas_differential_indexs(struct pic * const p, char * const name);
int pic_save_differential_indexs(struct pic * const p, FILE * const f);

/* Free memory */
void pic_destroy(struct pic * const p);
#endif