/*! 
 * \file utils.h
 * \brief Utilities
 *
 * \author Joan Gimeno
 * \date 12/11/2013 (start)
 */

#ifndef UTILS_H
#define UTILS_H

/*!
 * \def NDEBUG
 * \brief If the symbolic constant NDEBUG is defined, that is no debugging, subsequent assertions and some prints will be ignored
 */
// #define NDEBUG       /* Assert case */

/*! \def DEBUG_PRINT
 *  \brief Macro for printing information during the execution
 */
//#define DEBUG_PRINT     /* Prints case */

/*! \def DEBUG_QR_PRINT
 *  \brief Macro for printing information during the execution of QR decomposition
 */
//#define DEBUG_QR_PRINT     /* Prints case */

// #include <errno.h>
#include <stdlib.h>
// #include <stdio.h>
// #include <string.h>
// #include <math.h>
#include <ctype.h>
#include <assert.h>

#include "load.h"
#include "save.h"

#ifdef _OPENMP
  #include <omp.h>
#endif

#if __STDC_VERSION__ >= 199901L
/* "inline" is a keyword */
#else
#define inline static
#endif

/* Actually M_PI seems to be more widely used so we deprecate PI */
#ifndef PI
#define PI    3.1415926535897932384
#endif

#ifndef M_PI
#define M_PI    3.1415926535897932384
#endif

/*!
 * \def OPENMP_DIMENSION
 * \brief A macro that gives the minimum dimension of some vector or matrix that openmp will be used
 * \showinitializer
 */
#ifdef _OPENMP
#define _OPENMP_DIMENSION 1e2
#endif

/* Various utility macros */
/*!
 * \def MAX( a, b )
 * \brief A macro that gives the maximum of \a a and \a b
 * \showinitializer
 */
#ifndef MAX
#define MAX(a, b) ( ( ( a ) > ( b ) ) ? ( a ) : ( b ) )
#endif

/*!
 * \def MIN( a, b )
 * \brief A macro that gives the minimum of \a a and \a b
 * \showinitializer
 */
#ifndef MIN
#define MIN(a, b) ( ( ( a ) > ( b ) ) ? ( b ) : ( a ) )
#endif

/*!
 * \def ROUND( a )
 * \brief A macro that gives the round of \a a
 * \showinitializer
 */
#ifndef ROUND
#define ROUND( a ) ( ( a ) < 0. ? ( ( a ) - .5 ) : ( ( a ) + .5 ) )
#endif

/*!
 * \def ABS( a )
 * \brief A macro that gives the absolute value of \a a
 * \showinitializer
 */
#ifndef ABS
#define ABS( a ) ( ( a ) < 0. ? -( a ) : ( a ) )
#endif

/*!
 * \def SIGN( a )
 * \brief A macro that gives the signed of \a a
 * \showinitializer
 */
#ifndef SIGN
#define SIGN( a ) ( ( a ) < 0. ? -1 : 1 )
#endif

/*!
 * \def SWAP( a, b )
 * \brief A macro that swaps two generic values
 * \showinitializer
 */
#define SWAP( a, b ) typeof(a) _SWAPPING_VAR; _SWAPPING_VAR = a; a = b; b = _SWAPPING_VAR

/*!
 * \def free_mem( a )
 * \brief Used to help ensure everything malloc'ed gets freed
 * \hideinitializer
 */
#define free_mem( a ) \
if ( a != NULL ) { free( (void *) a ); a = NULL; }

#ifndef NULL
#define NULL (void *) 0
#endif

/* Function prototypes */
double euclidian_norm(size_t n, double * const x);
double inner_product(size_t n, double * const x);

void copy_vector(size_t n, double * const x, double * const y);
void copy_matrix(size_t m, size_t n, double ** const A, double ** const B);

void print_matrix(size_t m, size_t n, double ** const A, char * const name);
void print_vector(size_t idx, size_t n, double * const v, char * const name);

double * malloc_vector(size_t n);
void free_vector(double *p);
double ** malloc_matrix(size_t m, size_t n);
void free_matrix(size_t m, double **p);

int qrsis(int n, int m, double ** a, double * b, double tol); //TODO

#ifdef MEMORY_INFO
void add_bytes(size_t b);
void print(void);
void free_bytes(size_t b);
#endif

#ifdef MEMORY_INFO
/*!
 * \struct memory
 * \brief Contains information of memory allocated and freee
 */
struct memory {
  size_t used; /*! \var Megabytes of memory allocated */
  size_t free; /*! \var Megabytes of memory free */
};

struct memory m;
#endif
#endif