/*! 
 * \file load.c
 * \brief Implements functions that load a vector or a matrix in some file
 *
 * \author Joan Gimeno
 * \date 22/05/2014 (start)
 * \version 1.0
 */
#include "load.h"

/*!
 * \fn int load_vector(double * const b, FILE * const f)
 * \brief Function that loads the values of a vector in a given file
 * 
 * \param[in] b Pointer to a vector
 * \param[in] f Output file
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int load_vector(double * const b, FILE * const f) {  
  size_t idx;
  double d;
  
  while(fscanf(f, "%zu %lf", &idx, &d) != EOF)
    b[idx] = d;
    
  return 1;
}

/*!
 * \fn int loadas_vector(double * const b, char * const name)
 * \brief Function that loads the values of a vector in a given filename
 * 
 * \param[in] b Pointer to a vector
 * \param[in] name Filename
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int loadas_vector(double * const b, char * const name) {  
  int r;
  FILE *f;
  
  if ((f = fopen(name, "r")) == NULL)
    return 0;
  
  r = load_vector(b, f);
    
  return (fclose(f) != EOF) && (r == 1);
}

/*!
 * \fn int load_matrix(double ** const A, FILE * const f)
 * \brief Function that loads the values of a matrix in a given file
 * 
 * \param[in] A Pointer to a matrix
 * \param[in] f Output file
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int load_matrix(double ** const A, FILE * const f) {  
  size_t row, col;
  double d;
  
  while(fscanf(f, "%zu %zu %lf", &row, &col, &d) != EOF)
    A[row][col] = d; 
      
  return 1;
}

/*!
 * \fn int loadas_matrix(double ** const A, char * const name)
 * \brief Function that loads the values of a matrix in a given filename
 * 
 * \param[in] A Pointer to a matrix
 * \param[in] name Filename
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int loadas_matrix(double ** const A, char * const name) {  
  int r;
  FILE *f;
  
  if ((f = fopen(name, "r")) == NULL)
    return 0;
  
  r = load_matrix(A, f);
  
  return (fclose(f) != EOF) && (r == 1);
}

// #include <stdlib.h>
// int main() {
//   double **A;
//   int i, j, m, n;
//   m = n = 10;
//   
//   
//   if ((A = (double **) malloc(m*sizeof(double *))) == NULL)
//     return 1;
//   
//   for (i = 0; i < m; i ++)
//     if ((A[i] = (double *) calloc(n, sizeof(double))) == NULL) {
//       for (j = 0; j < i; j ++)
//         free(A[i]);
//       return 1;
//     }
//     
//   loadas_matrix(A, "test_matrix.txt");
//   
//   for (i = 0; i < m; i ++) {
//     for (j = 0; j < n; j ++)
//       printf("%f ", A[i][j]);
//     printf("\n");
//   }
//     
//   for (i = 0; i < m; i ++)
//     free(A[i]);
//   free(A);
//   return 0;
// }