/*! 
 * \file newton.c
 * \brief Implements the functions for a Newton's method 
 *
 * \author Joan Gimeno
 * \date 22/01/2014 (start)
 * \version 1.0
 */

#include "newton.h"

#ifdef _NEWTON_DEBUG
#define _MAX_STRING_LENGTH 50
#endif

/*!
 * \def _DEFAULT_NEWTON_MAX_ITERATIONS
 * \brief Maximum number of newton iterations
 * \showinitializer
 */
#define _DEFAULT_NEWTON_MAX_ITERATIONS 10

/*!
 * \var newton_max_iterations
 * \brief Maximum number iterations for Newton's method
 */
static unsigned int newton_max_iterations = _DEFAULT_NEWTON_MAX_ITERATIONS;

/*!
 * \fn unsigned int get_newton_max_iterations()
 * \brief Get the maximum number of newton iterations
 * 
 * \return Maximum number iterations for Newton's method
 */
unsigned int get_newton_max_iterations() {
  return newton_max_iterations;
}

/*!
 * \fn unsigned int set_newton_max_iterations(unsigned int newton_max_iter)
 * \brief Set the maximum number of newton iterations
 * 
 * \param[in] newton_max_iter New value
 * 
 * \return The old maximum number iterations for Newton's method
 */
unsigned int set_newton_max_iterations(unsigned int newton_max_iter) {
  unsigned int tmp;
  
  tmp = newton_max_iterations;
  newton_max_iterations = newton_max_iter;
  
  return tmp;
}

double * malloc_vector2(size_t n) {
  #ifdef _NEWTON_DEBUG
  char str[_MAX_STRING_LENGTH];
  #endif
  double *p;
  
  if ((p = (double *) malloc(n * sizeof(double))) == NULL) {
    #ifdef _NEWTON_DEBUG
    sprintf(str, "Unsuccess allocated memory for a %zu-dimensional vector", n);
    perror(str);
    #endif
    return NULL;
  }
  return p;
}

// pic_newton(m + p->autonomous, m, p->X, h, p->DF, p->solve, p->x, tol)
// pic_newton(m + p->autonomous, m, p->X, p->x, p->DF, p->solve, h, tol)
/*!
 * \fn int newton(size_t m, size_t n, double * const x, double * const f, double ** const df, void * (*solve)(size_t, size_t, double ** const, double * const, double * const, double), double tol)
 * \brief Implements the compute of a newton method's iteration
 * 
 * \param[in] m Rows of the matrix
 * \param[in] n Columns of the matrix
 * \param[in] x Current iteration value
 * \param[out] x Next iteration value
 * \param[in] f Constant terms
 * \param[in] df Matrix of the linear system
 * \param[in] *solve Pointer to solver linear systems of equations
 * \param[in] tol Tolerance
 * 
 * \retval 0 Some problem has ocurred
 * \retval 1 When the solution x is accepted by the tolerance given 
 */
int newton(size_t m, 
           size_t n, 
           double * const x, 
           double * const f, 
           double ** const df,
           void * (*solve)(size_t, size_t, double ** const, double * const, double * const, double),
           double tol) {
  typeof(n) i;
  typeof(*x) aux, error;
  
  error = fabs(f[0]);
  for (i = 1; i < n; i ++) {
    aux = fabs(f[i]);
    if (error < aux) error = aux;
  }  
  
  #ifdef _NEWTON_DEBUG
  printf("pic_newton: error = %7.5e tolerance = %7.5e\n", error, tol);
  #endif
  if (error <= tol) return 1;
  
  if (solve(m, n, df, f, f, tol) == NULL) {
    //   { double ** adf, *af; adf = df; af = f;
    //   if (qrsis(m, n, adf, af, tol) == 0) {
      #ifdef _NEWTON_DEBUG
      printf("pic_newton: Unsuccessful solving linear system\n");
      #endif
      return 0;
      //   }
  }
  
  for (i = 0; i < n; i ++)
    x[i] -= f[i];
  return 0;
  
  error = fabs(f[0]);
  x[0] -= f[0];
  
  /* Update de solution x and compute the error in infinity norm */
  for (i = 1; i < n; i ++) {
    aux = fabs(f[i]);
    if (error < aux) error = aux;
    
    x[i] -= f[i];
  }
  
  #ifdef _NEWTON_DEBUG
  printf("pic_newton: error = %7.5e tolerance = %7.5e and remainder %7.5e\n", error, tol, f[m-1]);
  #endif
  if (error <= tol) return 1;
  
  return 0;
}

/*!
 * \fn void newton_differential_block(size_t m, size_t n, double * const x, double * const f, double ** const df, double w, double t)
 * \brief Compute a block of the differential for the Newton's method
 * 
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in] x 
 * \param[in] f 
 * \param[out] f 
 * \param[out] df Block matrix of the differential for the Newton's method
 * \param[in] w Parameter
 * \param[in] t Angle of evaulation
 * 
 * \pre The pointers x and f are not NULL
 * 
 * \note If df is NULL, the differential will not be compute
 */
void newton_differential_block(size_t m, 
                               size_t n, 
                               double * const x, 
                               double * const f, 
                               double ** const df, 
                               double w, 
                               double t) {
  typeof(n) i, j, k, l;
  double c, s;
  
  for (i = 0; i < m; i ++)
    f[i] -= x[i];
  
  if (df == NULL) return;
  
  /* Rows */
  for (i = 0; i < m; i ++) { 
    l = 1;
    /* Columns */
    for (j = 1; j < n; j += 2) { 
      c = cos(l * t);
      s = sin(l * t);
      
      /* Lower */
      for (k = 0; k < i; k ++) {
        df[i][k + j*m] = df[i][k] * c;
        df[i][k + (j+1)*m] = df[i][k] * s;
      }
      
      /* Diagonal */
      df[i][i + j*m] = df[i][i] * c - cos(l * (t + w));
      df[i][i + (j+1)*m] = df[i][i] * s - sin(l * (t + w));
      
      /* Upper */
      for (k = i + 1; k < m; k ++) { 
        df[i][k + j*m] = df[i][k] * c;
        df[i][k + (j+1)*m] = df[i][k] * s;
      }
      l ++;
    }
    df[i][i] -= 1.0;
  }
}

// dft_r2r_1d(n, N, X, h, t * i);
/*!
 * \fn void dft_r2r_1d(size_t n, size_t N, double * const dft, double * const x, double theta)
 * \brief Compute the Discrete Fourier Transform
 * 
 * \param[in] n Space's dimension
 * \param[in] N Mesh size
 * \param[out] x Discrete Fourier Transform
 * \param[in] X Fourier coefficients
 * \param[in] theta Angle evaluation
 */
void dft_r2r_1d(size_t n, size_t N, double * const dft, double * const x, double theta) {
  typeof(n) i, j, k;  
  double c, s;
  
  N = 2 * N + 1;
//   printf("dft_r2r_1d %zu %zu \n", n, N);
  for (j = 0; j < n; j ++)
    x[j] = dft[j];
  
  k = 1;
  for (i = 1; i < N; i += 2) {
    c = cos(k * theta);
    s = sin(k * theta);
    
    for (j = 0; j < n; j ++) 
      x[j] += dft[j + i*n] * c + dft[j + (i+1)*n] * s;
    
    k ++;
  }
}

// pic_DF(p->n, p->N, p->x, p->X, p->DF, p->f, p->w, h);
/*!
 * \fn void newton_differential(size_t n, size_t N, double * const x, double * const X, double ** const DF, void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double), double w, double * h)
 * \brief Compute the differential for the Newton's method
 * 
 * \param[in] n Space's dimension
 * \param[in] N Mesh size
 * \param[in,out] x Discrete Fourier Transform for each mesh's element, that is, n(2N+1) components
 * \param[in] X Fourier coefficients
 * \param[in] DF Matrix of the linear system
 * \param[in] f Pointer to function given by the user
 * \param[in] w Parameter
 * \param[in] h Auxiliar vector
 * 
 * \return Number of iterations have been performed or NULL if some problem has ocurred
 */
void newton_differential(size_t n, 
                         size_t N, 
                         double * const x, 
                         double * const X, 
                         double ** const DF, 
                         void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double),
                         double w, 
                         double * h) {
  size_t m, i;
  double t;
  short int flag = 0;  
  
  m = 2 * N + 1;
  t = 2 * M_PI / m;
  
  if (h == NULL) {
    if ((h = malloc_vector(n*m)) == NULL)
      return;
    flag = 1;
  }
//   printf("\t\t0\n");
  
  #pragma omp parallel if(m > _OPENMP_NEWTON_METHOD) 
  {
  #pragma omp for private(i)
  for (i = 0; i < m; i ++) {
//     printf("\t\t1");
    dft_r2r_1d(n, N, X, h + i*n, t * i);
    
//     printf("\t\t2");
    f(n, n, h + i*n, x + i*n, DF + i*n, w, t * i);
    
//     printf("\t\t3");
    dft_r2r_1d(n, N, X, h + i*n, t * i + w);
    
//     printf("\t\t4\n");
    newton_differential_block(n, m, h + i*n, x + i*n, DF + i*n, w, t * i);
  }
  }
  
  //TODO
//   m = n * m;
//   if (autonomous) {
//     for (i = 0; i < m; i ++) {
//       DF[m][i] = DF[0][i];
//     }
//     x[m] = x[0];
//   }
  //
  
  if (flag) free(h);
}

/*!
 * \fn void * newton_execute(size_t n, size_t N, double * const x, double * const X, double ** const DF, double w, double tol, void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double), void * (*solve)(size_t, size_t, double ** const, double * const, double * const, double), short unsigned int autonomous)
 * \brief Implement the execution of a Newton's method
 * 
 * \param[in] n Space's dimension
 * \param[in] N Mesh size
 * \param[in,out] x Discrete Fourier Transform for each mesh's element, that is, n(2N+1) components
 * \param[in] X Fourier coefficients
 * \param[in] DF Matrix of the linear system
 * \param[in] w Parameter
 * \param[in] tol Tolerance
 * \param[in] f Pointer to function given by the user
 * \param[in] solve Pointer to solver linear systems of equations
 * 
 * \return Number of iterations have been performed or NULL if some problem has ocurred
 * 
 * \see \ref newton_max_iterations 
 */
void * newton_execute(size_t n, size_t N, double * const x, double * const X, double ** const DF, 
                      double w, double tol,
                      void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double),
                      void * (*solve)(size_t, size_t, double ** const, double * const, double * const, double),
                      short unsigned int autonomous
                     ) {
  size_t k, m;
  
  m = n * (2 * N + 1);
  
  double *h = NULL;
  
  if ((h = (double *) malloc(m * sizeof(double))) == NULL) {
    #ifdef _NEWTON_DEBUG
    perror("NEWTON_EXECUTE: Unsuccess allocated memory for auxiliar vector");
    #endif
    return NULL;
  }
  
//   printf("\t\t-1\n");
  for (k = 0; k < newton_max_iterations; k ++) {
    newton_differential(n, N, x, X, DF, f, w, h);
    
    if (newton(m + autonomous, m, X, x, DF, solve, tol) != 0)
      break;
  }
  
  free(h);
  
  return (void *) k;  
}