/*!
 * \file permutation.c
 * \brief Permutation operation with a vector 
 * 
 * \author Joan
 * \date 20/01/2014 (start)
 * \version 1.0
 */

/*!
 * \defgroup perm Vector permutation
 * \brief The act of permuting objects or values is called permutation and we implement a permutation to a vector
 * 
 * A permutation \f$ \tau \f$ of \f$ n+1 \f$ elements is a bijection of \f$ \{0,\ldots,n\} \f$ from that set onto itself and with the composition is a non-commutative group. The Cauchy's two-line notation is 
 * 
 * \f[ \tau = \begin{pmatrix} 0 & \cdots & n \\ \tau(0) & \cdots & \tau(n) \end{pmatrix}. \f]
 * 
 * In one-line notation, one gives only the second row of this array.
 * In this functions, one-line notation will be used.
 * @{
 */

#include "permutation.h"

/*!
 * \def _OPENMP_PERMUTATION_DIMENSION
 * \brief Minimum value for using the openmp library in permutation
 */
#define _OPENMP_PERMUTATION_DIMENSION 1e2

/*!
 * \fn void permutation_init(size_t n, size_t * const p)
 * \brief Initialize a identity vector permutation
 * 
 * 
 * \param[in] n Dimension of vector
 * \param[out] p Vector permutation
 * 
 * \pre Dimension is not zero
 * \note If vector permutation is NULL, anything is done
 */
void permutation_init(size_t n, size_t * const p) {
  assert(n != 0);  
  if (p == NULL) return;
  
  typeof(n) i;
  
  #pragma omp parallel if (n > _OPENMP_PERMUTATION_DIMENSION) default(shared)
  {
  #pragma omp for private(i) nowait
  for (i = 0; i < n; i ++)
    p[i] = i;
  }
}

/*!
 * \fn void permutation_vector(size_t n, size_t * const p, double * const x, double * const px)
 * \brief Permute the values of the vector x into px based on the vector permutation p
 * 
 * \param[in] n Dimension of vectors
 * \param[in] p Vector permutation
 * \param[in] x Vector n-dimensional where will be read the data
 * \param[out] px Vector n-dimensional where will be saved the permutations by the vector permutation
 * 
 * \pre Dimension is not 0 and the pointers x and px are not NULL
 * 
 * \note The vectors can be the same
 * If the vector permutation is NULL, anything is done
 */
void permutation_vector(size_t n, size_t * const p, double * const x, double * const px) {
  assert(n != 0);
  if (p == NULL) return;
  
  typeof(n) i, j;
  typeof(*x) tmp;
  
  for (i = 0; i < n; i ++) {
    j = p[i];
    while (j < i) j = p[j];
    
    /* Swap values */
    tmp = x[i];
    px[i] = x[j];
    px[j] = tmp;
  }
}
/*!
 * @}
 */