/*! 
 * \file pic.c
 * \brief Implements differents functions for the pic struct 
 *
 * \author Joan Gimeno
 * \date 22/01/2014 (start)
 * \version 1.0
 */

#include "pic.h"

#ifdef _PIC_DEBUG
#define _MAX_STRING_LENGTH 50
#endif

/*!
 * \def _DEFAULT_MAX_ITERATIONS
 * \brief Maximum number of newton iterations
 * \showinitializer
 */
#define _DEFAULT_MAX_ITERATIONS 80

/*!
 * \var max_iterations
 * \brief Maximum iterations that increase the mesh size
 */
static unsigned int max_iterations = _DEFAULT_MAX_ITERATIONS;

/*!
 * \fn unsigned int pic_get_max_iterations()
 * \brief Get maximum number of iterations
 * 
 * \return Maximum iterations that increase the mesh size
 */
unsigned int pic_get_max_iterations() {
  return max_iterations;
}

/*!
 * \fn unsigned int pic_set_max_iterations(unsigned int max_iter)
 * \brief Set maximum number of iterations
 * 
 * \param[in] max_iter New value
 * 
 * \return The old maximum iterations that increase the mesh size
 */
unsigned int pic_set_max_iterations(unsigned int max_iter) {
  unsigned int tmp;
  
  tmp = max_iterations;
  max_iterations = max_iter;
  
  return tmp;
}

/*!
 * \fn size_t pic_get_mesh_size(struct pic * const p)
 * 
 * \param[in] p Pointer to a pic struct
 * 
 * \return Mesh size of the pic struct
 * 
 * \pre The pointer is not NULL
 */
size_t pic_get_mesh_size(struct pic * const p) {
  return p->N;
}

/*!
 * \fn size_t pic_get_dimension_size(struct pic * const p)
 * 
 * \param[in] p Pointer to a pic struct
 * 
 * \return Space's dimension of the pic struct
 * 
 * \pre The pointer is not NULL
 */
size_t pic_get_dimension_size(struct pic * const p) {
  return p->n;
}

/*!
 * \fn int pic_set_function_pointer(struct pic * const p, _FUNCTION_PROTOTYPE)
 * 
 * \param[in] p Pointer to a pic struct
 * \param[in] f Pointer to a function pointer
 * 
 * \retval 0 if some argument is NULL
 * \retval 1 Otherwise
 */
int pic_set_function_pointer(struct pic * const p, 
                             void (*f)(size_t, size_t, double * const, double * const, double ** const, double, double)) {
  if (p == NULL || f == NULL) return 0;
  
  p->f = f;
  return 1;
}

/*!
 * \fn int pic_set_function_solve(struct pic * const p, _SOLVE_PROTOTYPE)
 * 
 * \param[in] p Pointer to a pic struct
 * \param[in] f Pointer to a function pointer
 * 
 * \retval 0 if some argument is NULL
 * \retval 1 Otherwise
 */
int pic_set_function_solve(struct pic * const p, _SOLVE_PROTOTYPE) {
  if (p == NULL || solve == NULL) return 0;
  
  p->solve = solve;
  return 1;
}

//  * \param[in] autonomous Indicates if the dynamic system is autonomous or not. Only it can be  either \ref PIC_AUTONOMOUS (default) or \ref PIC_NAUTONOMOUS
/*!
 * \fn struct pic * pic_create(size_t n, size_t N, double w, double * const X, _FUNCTION_PROTOTYPE, _SOLVE_PROTOTYPE)
 * \brief Function that initializes a struct pic
 * 
 * \param[in] n Space's dimension of the function \f$ f \colon \mathbb{R}^n \rightarrow \mathbb{R}^n \f$
 * \param[in] N Mesh size of the \f$ \mathbb{T}^1 \f$, 2N+1
 * \param[in] w Real number densely filling the circumference
 * \param[in] X Seed of the newton method. The size has to be n(2N+1)
 * \param[in] f Function pointer
 * \param[in] solve Function pointer for solves a linear system equations
 * 
 * \retval NULL If memory space has not been allocated
 * \retval p Pointer to struct \ref pic
 * 
 * \pre The space's dimension is not zero
 * \post The struct pic has been initialized
 * \note If the seed of newton method is NULL, the default seed will be the vector zero
 */
struct pic * pic_create(size_t n, size_t N, double w, double * const X, _FUNCTION_PROTOTYPE, _SOLVE_PROTOTYPE) {
  assert(n > 0);
  #ifdef _PIC_DEBUG
  printf("PIC: n = %zu N = %zu  w = %7.5e autonomous = %u\n", n, N, w, PIC_NAUTONOMOUS);
  #endif
  
  struct pic *p;
  if ((p = (struct pic *) malloc(sizeof(struct pic))) == NULL) {
    #ifdef _PIC_DEBUG
    perror("Unsuccess allocated memory struct pic");
    #endif
    return NULL;
  }
  
  p->n = n;
  p->N = N;
  p->w = w;
  p->autonomous = PIC_NAUTONOMOUS;
  p->x = NULL;
  p->X = X;
  p->DF = NULL;
  
  p->f = f;
  p->solve = solve ? solve: SOLVE_WITH_LU;
  
  return p;
}

/*!
 * \fn void * _pic_realloc(struct pic * const p, size_t n, size_t N, short unsigned int autonomous)
 * \brief
 * 
 * \param[in] p Pointer to a pic struct
 * \param[in] n Space's dimension of the function \f$ f \colon \mathbb{R}^n \rightarrow \mathbb{R}^n \f$
 * \param[in] N Mesh size of the \f$ \mathbb{T}^1 \f$, 2N+1
 * \param[in] autonomous
 * 
 * \retval NULL If memory space has not been reallocated or the pointer to a pic struct is NULL
 * \retval 1 Otherwise
 * 
 * \pre The mesh size and space's dimension are not zeros
 * \post
 *      - The \ref p::x,\ref p::X and \ref p::DF have been changed the size of the memory block pointed
 *      - The field \ref pic::N and \ref pic::n have been assigned with the new values given by parameter
 * \note
 *      - The new positons in \ref pic::X will be assigned with zero values
 *      - The new rows of \ref pic::DF will be allocated
 */
void * _pic_realloc(struct pic * const p, size_t n, size_t N, short unsigned int autonomous) {
  if (p == NULL) return NULL;
  
  typeof(n) m, i, j;
  
  #ifdef _PIC_DEBUG
  char str[_MAX_STRING_LENGTH];
  #endif
  
  m = n * (2 * N + 1);          /* New dimension */
  j = p->n * (2 * p->N + 1);    /* Previous dimension */
  
  if ((p->x = (double *) realloc(p->x, (m + autonomous) * sizeof(double))) == NULL) {
    #ifdef _PIC_DEBUG
    perror("Unsuccess reallocated memory for the vector x");
    #endif
    return NULL;
  }
  
  if ((p->X = (double *) realloc(p->X, m * sizeof(double))) == NULL) {
    #ifdef _PIC_DEBUG
    perror("Unsuccess reallocated memory for the vector X");
    #endif
    return NULL;
  }
  
  for (i = j; i < m; i ++)
    (p->X)[i] = 0.0; //NOTE That should be say in some moment
  
  if ((p->DF = (double **) realloc(p->DF, (m + autonomous) * sizeof(double *))) == NULL) {
    #ifdef _PIC_DEBUG
    perror("Unsuccess reallocated memory for the matrix DF");
    #endif
    return NULL;
  }
  
  for (i = 0; i < j; i ++)
    if (((p->DF)[i] = (double *) realloc((p->DF)[i], m * sizeof(double))) == NULL) {
      #ifdef _PIC_DEBUG
      snprintf(str, _MAX_STRING_LENGTH, "Unsuccess reallocated memory for the vector DF[%zu]", i);
      perror(str);
      #endif
      return NULL;
    }
  for ( ; i < m; i ++)
    if (((p->DF)[i] = (double *) malloc(m * sizeof(double))) == NULL) {
      #ifdef _PIC_DEBUG
      snprintf(str, _MAX_STRING_LENGTH, "Unsuccess allocated memory for the vector DF[%zu]", i);
      perror(str);
      #endif
      return NULL;
    }
    
  if (p->N != N) p->N = N;
  if (p->n != n) p->n = n;
  if (p->autonomous != autonomous) p->autonomous = autonomous;
  
  return (void *) 1;
}

/*!
 * \fn int pic_set_mesh_size(struct pic * const p, size_t N)
 * \brief Set the mesh size
 * 
 * \param[in] p Pointer to a pic struct
 * \param[in] N New mesh size
 * 
 * \return 0 if some error occurred or the old mesh size
 * 
 * \pre The new mesh size is not 0 and the pointer is not NULL
 * \post The pointer memory has been reallocated with the new mesh size
 */
int pic_set_mesh_size(struct pic * const p, size_t N) {
  if (p == NULL || N == 0) return 0;
  
  typeof(N) M;
  
  M = p->N;
  
  if (_pic_realloc(p, p->n, N, p->autonomous) == NULL)
    return 0;
  
  if (p->N != N)
    p->N = N;
  
  return M;
}

/*!
 * \fn int pic_set_dimension_size(struct pic * const p, size_t n)
 * \brief Set the dimension
 * 
 * \param[in] p Pointer to a pic struct
 * \param[in] N New dimension
 * 
 * \return 0 if some error occurred or the old dimension
 * 
 * \pre The new mesh size is not 0 and the pointer is not NULL
 * \post The pointer memory has been reallocated with the new dimension
 */
int pic_set_dimension_size(struct pic * const p, size_t n) {
  if (p == NULL || n == 0) return 0;
  
  typeof(n) M;
  
  M = p->n;
  
  if (_pic_realloc(p, n, p->N, p->autonomous) == NULL)
    return 0;
  
  if (p->n != n)
    p->n = n;
  
  return M;
}

/*
 * \fn void * _init_pic(struct pic * const p)
 * \brief Allocate memory for computing a periodic invariant curve (\ref pic)
 * 
 * \param[in] p Pointer to periodic invariant curve
 * 
 * \retval NULL If it cannot be allocated memory for some pointer
 * \retval 1 Otherwise
 * 
 * \pre The \ref pic::n, \ref pic::N and \ref pic::autonomous have been initialized
 */
void * _init_pic(struct pic * const p) {
  if (p == NULL) return NULL;
  size_t m;
  
  m = p->n * (2 * p->N + 1);
                                   
  if (p->x == NULL)
    if ((p->x = (double *) malloc((m + p->autonomous) * sizeof(double))) == NULL) {
      #ifdef _PIC_DEBUG
      perror("Unsuccess allocated memory for the vector x");
      #endif
      return NULL;
    }
  
  if (p->X == NULL)
    if ((p->X = (double *) calloc(m, sizeof(double))) == NULL) {
      #ifdef _PIC_DEBUG
      perror("Unsuccess allocated memory for the vector X");
      #endif
      free(p->x); p->x = NULL;
      return NULL;
    }

  if (p->DF == NULL) {
    size_t i, j;
    
    if ((p->DF = (double **) malloc((m + p->autonomous) * sizeof(double *))) == NULL) {
      #ifdef _PIC_DEBUG
      perror("Unsuccess allocated memory for differential matrix DF");
      #endif
      free(p->X); free(p->x);
      p->x = p->X = NULL;
      return NULL;
    }
  
    j = m + p->autonomous;
    for (i = 0; i < j; i ++)
      if (((p->DF)[i] = (double *) malloc(m * sizeof(double))) == NULL) {
        #ifdef _PIC_DEBUG
        perror("Unsuccess allocated memory for differential matrix DF");
        #endif
        
        for (j = 0; j < i; j ++)
          free((p->DF)[j]);
        
        free(p->X); free(p->x); free(p->DF);
        p->x = p->X = NULL;
        p->DF = NULL;
        return NULL;
      }
  }
  return (void *) 1;
}

/*!
 * \fn void pic_destroy(struct pic *p)
 * \brief Function that frees the memory space pointed to by the differents pointers of struct pic
 * 
 * \param[in] p Pointer to periodic invariant curve
 * 
 * \post The memory space has been freed
 */
void pic_destroy(struct pic *p) {
  if (p == NULL) return;
  
  free(p->x); p->x = NULL;
  free(p->X); p->X = NULL;
  
  if (p->DF) {
    size_t m, i;
    m = p->n * (2 * p->N + 1) + p->autonomous;
    
    for (i = 0; i < m; i ++) { 
      free((p->DF)[i]); 
      (p->DF)[i] = NULL; 
    }
  }
  
  free(p->DF); p->DF = NULL;
  free(p); p = NULL;
  
  #ifdef _PIC_DEBUG
  fprintf(stdout, "Free pic memory\n");
  #endif
}

int _get_fourier_coef_index(char ** const c) {
  while (isblank(**c)) (*c) ++;

  int k = atoi(*c);

  if (**c == '-') (*c) ++;
  while (isdigit(**c)) (*c) ++;
  while (isblank(**c)) (*c) ++;
  
  return k;
}

#define _MAX_LENGTH_STRING_INPUT 128

/*!
 * \fn double * pic_load_fourier_coef(size_t n, size_t N, FILE * const f)
 * \brief Function that load the Fourier coefficients from a given file
 * 
 * \param[in] n Dimension of each coefficient
 * \param[in] N Number of coefficient
 * \param[in] f Pointer to a file
 * 
 * \return n*(2N+1)-dimensional vector with the Fourier coefficients saved
 * 
 * \pre The pointer is not NULL
 * \post The user has to free the memory for the pointer returned
 * 
 * \note The structure of the vector is a_0, a_1, b_1, ..., a_N, b_N where each element is a n-dimensional vector
 */
double * pic_load_fourier_coef(size_t n, size_t N, FILE * const f) {
  size_t i, k, m;
  
  m = 2 * N + 1;
  
  double *dft;  
  if ((dft = (double *) calloc(m*n, sizeof(double))) == NULL)
    return NULL;
  
  char s[_MAX_LENGTH_STRING_INPUT], *c, *p;
  
  while(fgets(s, _MAX_LENGTH_STRING_INPUT, f) != NULL) {
    c = s;
    
    if (*c != '\0') {
      while (isblank(*c)) c ++;
      
      if (*c == 'a') {
        c ++;
        k = strtoul(c, &p, 10);//_get_fourier_coef_index(&c);
        i = strtoul(p, &c, 10);
        
        if (k == 0)
          dft[i] = strtod(c, NULL); //atof(p);
        else
          dft[i + k*n*n - n] = strtod(c, NULL); //atof(p);
        
      } else if (*c == 'b') {
        c ++;
        k = strtoul(c, &p, 10);//_get_fourier_coef_index(&c);
        i = strtoul(p, &c, 10);
        
        dft[i + k*n*n] = strtod(c, NULL); //atof(p);
      }
    }
  }
  
  return dft;
  
  size_t *countA, *countB;  
  if ((countA = (size_t *) calloc(N, sizeof(size_t))) == NULL)
    return NULL;
  if ((countB = (size_t *) calloc(N, sizeof(size_t))) == NULL) {
    free(countA);
    return NULL;
  }
  
  while(fgets(s, _MAX_LENGTH_STRING_INPUT, f) != NULL) {
    c = s;
    
    if (*c != '\0') {
      while (isblank(*c)) c ++;
      
      if (*c == 'a') {
        c ++;
        k = strtoul(c, &p, 10);// _get_fourier_coef_index(&c);
        if (k == 0)
          dft[countA[k]] = strtod(p, &c); //atof(p);
        else
          dft[countA[k] + k*n*n - n] = strtod(p, &c); //atof(p);
        
        countA[k] = (countA[k] + 1) % n;
      } else if (*c == 'b') {
        c ++;
        k = strtoul(c, &p, 10);//_get_fourier_coef_index(&c);
        
        dft[countB[k] + k*n*n] = strtod(p, &c); //atof(p);
        
        countB[k] = (countB[k] + 1) % n;
      }
    }
  }
  
  free(countA); free(countB);
  return dft;
}


/*!
 * \fn double * pic_loadas_fourier_coef(size_t n, size_t N, char * const name)
 * \brief Function that load the Fourier coefficients from a given file by name
 * 
 * \param[in] n Dimension of each coefficient
 * \param[in] N Number of coefficient
 * \param[in] f Pointer to a file
 * 
 * \return n*(2N+1)-dimensional vector with the Fourier coefficients saved
 * \post The user has to free the memory for the pointer returned
 * 
 * \pre The pointer is not NULL
 * 
 * \note The structure of the vector is a_0, a_1, b_1, ..., a_N, b_N where each element is a n-dimensional vector
 */
double * pic_loadas_fourier_coef(size_t n, size_t N, char * const name) {
  FILE *f;
  double *dft;
  
  if ((f = fopen(name, "r")) != NULL) {
    dft = pic_load_fourier_coef(n, N, f);
    
    if (fclose(f) == EOF) {
      free(dft);
      dft = NULL;
    }
    
  } else {
    dft = NULL;
  }
  
  return dft;
}

/*!
 * \fn int pic_save_fourier_coef(struct pic * const p, FILE * const f)
 * \brief Function that saves the Fourier coefficients in a given file
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] f Output file 
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file has been created or modified with the result
 */
int pic_save_fourier_coef(struct pic * const p, FILE * const f) {  
  size_t j, k, n, N;
  
  n = p->n;
  N = p->N + 1;
  
  /*
   * Header of the file
   */
  if (fprintf(f, "Coef\tIndex\tValue\n") < 0)
    return 0;
  
  /*
   * Table of values
   */
  // Case 0
  k = 0;
  double *X = p->X;
  
  // Coefficient a_0
  for (j = 0; j < n; j ++, X ++)
    if (fprintf(f, "a %zu %zu %24.16e\n", k, j, *X) < 0)
      return 0;
  
  for (k = 1; k < N; k ++) {
    // Coefficient a_k
    for (j = 0; j < n; j ++, X ++)
      if (fprintf(f, "a %zu %zu %24.16e\n", k, j, *X) < 0)
        return 0;
      
    // Coefficient b_k
    for (j = 0; j < n; j ++, X ++)
      if (fprintf(f, "b %zu %zu %24.16e\n", k, j, *X) < 0)
        return 0;
  }
  
  return 1;
  
//   size_t i;
//   // Case 0
//   k = 0;
//   for (j = 0; j < n; j ++)
//     if (fprintf(f, "a%zu\t%16e\n", k, (p->X)[j]) < 0)
//       return 0;
//     
//     k = 1;
//   
//   for (i = 1; i < m; i ++) {    
//     for (j = 0; j < n; j ++)
//       if (fprintf(f, "a%zu\t%16e\n", k, (p->X)[i*n + j]) < 0)
//         return 0;
//     i ++;
//     for (j = 0; j < n; j ++)
//       if (fprintf(f, "b%zu\t%16e\n", k, (p->X)[i*n + j]) < 0)
//         return 0;
//     k ++;
//   } 
//   
//   return 1;
//   
//   // Case 0
//   k = 0;
//   if (fprintf(f, "a%zu", k) < 0)
//     return 0;
//   for (j = 0; j < n; j ++)
//     if (fprintf(f, "\t%16e\n", (p->X)[j]) < 0)
//       return 0;
//   
//   k = 1;
//   
//   for (i = 1; i < m; i ++) {
//     if (fprintf(f, "a%zu", k) < 0)
//         return 0;
//     
//     for (j = 0; j < n; j ++)
//       if (fprintf(f, "\t%16e\n", (p->X)[i*n + j]) < 0)
//         return 0;
//       
//     if (fprintf(f, "b%zu", k) < 0)
//         return 0;
//     
//     i ++; k ++;
//     for (j = 0; j < n; j ++)
//       if (fprintf(f, "\t%16e\n", (p->X)[i*n + j]) < 0)
//         return 0;
//   }
//   
//   return 1;
}

/*!
 * \fn int pic_saveas_fourier_coef(struct pic * const p, char * const name)
 * \brief Function that saves the Fourier coefficients in a given file by name
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] f Output file 
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file has been created or modified with the result
 */
int pic_saveas_fourier_coef(struct pic * const p, char * const name) {
  FILE *f;
  
  if ((f = fopen(name, "w")) == NULL)
    return 0;
  
  int r = pic_save_fourier_coef(p, f);
  
  if (fclose(f) == EOF || r == 0) {
    #ifdef _PIC_DEBUG
    fprintf(stdout, "Unsuccessful saved the Fourier coefficients in %s\n", name);
    #endif
    return 0;
  }
  
  #ifdef _PIC_DEBUG
  fprintf(stdout, "Successful saved the Fourier coefficients in %s\n", name);
  #endif
  
  return 1;
}

/*!
 * \fn int pic_save_curve(struct pic * const p, FILE * const f)
 * \brief Function that saves the values of the solution curve in a given file
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] f Output file 
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int pic_save_curve(struct pic * const p, FILE * const f) {
  size_t i, j, m;
  double t, *x;
  
  m = 2 * (p->N) + 1;
  t = 2.0 * M_PI / m;
  
  /* Header of the file */
  if (fprintf(f, "%10s", "t") < 0)
    return 0;
  
  for (i = 0; i < p->n; i ++)
    if (fprintf(f, "\t%10s%zu", "x", i+1) < 0)
      return 0;
    
  /* Table of values */
  x = p->x;
  for (i = 0; i < m; i ++) {
    if (fprintf(f, "\n%24.16e", t*i) < 0)
      return 0;
    
    for (j = 0; j < p->n; j ++, x ++)
      if (fprintf(f, " %24.16e", *x) < 0) //(p->x)[j + i * p->n]
        return 0;
  }
  
  /* Close the curves with the first value */
  if (fprintf(f, "\n%24.16e", t*i) < 0)
    return 0;
  
  for (j = 0; j < p->n; j ++)
    if (fprintf(f, " %24.16e", (p->x)[j]) < 0)
      return 0;
    
  return 1;
}

/*!
 * \fn int pic_saveas_curve(struct pic * const p, char * const name)
 * \brief Function that saves the values of the solution curve in a given file by name
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] name Name of the output file 
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The struct pointer is not NULL
 * \post The file with name has been created or modified with the result
 */
int pic_saveas_curve(struct pic * const p, char * const name) {
  FILE *f;
  
  if ((f = fopen(name, "w")) == NULL)
    return 0;
  
  int r = pic_save_curve(p, f);
  
  if (fclose(f) == EOF || r == 0) {
    #ifdef _PIC_DEBUG
    fprintf(stderr, "Unsuccessful saved the solution curve in %s\n", name);
    #endif
    return 1;
  }
  
  #ifdef _PIC_DEBUG
  fprintf(stdout, "Successful saved the solution curve in %s\n", name);
  #endif
  
  return 0;
}

/*!
 * \fn int pic_saveas_differential(struct pic * const p, char * const name)
 * \brief Function that saves the values of the differential of Newton method in a given file by name
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] name Name of the output file 
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The struct pointer is not NULL
 * \post The file with name has been created or modified with the result
 */
int pic_saveas_differential(struct pic * const p, char * const name) {
  size_t m = p->n * (2 * p->N + 1);
  return saveas_matrix(0, m, 0, m, p->DF, name); 
}

/*!
 * \fn int pic_save_differential(struct pic * const p, FILE * const f)
 * \brief Function that saves the values of the differential of Newton method in a given file
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] f Output file 
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int pic_save_differential(struct pic * const p, FILE * const f) {
  size_t m = p->n * (2 * p->N + 1);
  return save_matrix(0, m, 0, m, p->DF, f); 
}

/*!
 * \fn int pic_saveas_differential_indexs(struct pic * const p, char * const name)
 * \brief Function that saves the values of the differential of Newton method in a given file by name from its indexs
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] name Name of the output file 
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The struct pointer is not NULL
 * \post The file with name has been created or modified with the result
 */
int pic_saveas_differential_indexs(struct pic * const p, char * const name) {
  size_t m = p->n * (2 * p->N + 1);
  return saveas_matrix_indexs(0, m, 0, m, p->DF, name); 
}

/*!
 * \fn int pic_save_differential_indexs(struct pic * const p, FILE * const f)
 * \brief Function that saves the values of the differential of Newton method in a given file from its indexs
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] f Output file 
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int pic_save_differential_indexs(struct pic * const p, FILE * const f) {
  size_t m = p->n * (2 * p->N + 1);
  return save_matrix_indexs(0, m, 0, m, p->DF, f); 
}

/*!
 * \fn void * pic_error(struct pic * const p, size_t N, size_t * const idx, double * const max)
 * \brief Function that compute the maximum error for a solution previously computed
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] N Mesh size that we want to check the error of the solution (e.g. 2*(p->N))
 * \param[out] idx Index where the maximum is reached
 * \param[out] max Maximum value of error
 * 
 * \retval NULL if some problem occurred
 * \retval 1 otherwise
 * 
 * \pre 
 *      - The struct pointer, the index and the maximum are not NULL
 *      - The fields of the pointer to struct pic are not NULL
 * \post The idx and max have been changed
 */
void * pic_error(struct pic * const p, size_t N, size_t * const idx, double * const max) {
  if (p == NULL) return NULL;
  if (idx == NULL || max == NULL) return NULL;
  
  size_t i, j, k;
  double error, aux, t;
  double *h;
  
  h = NULL;
  if ((h = malloc_vector(2*p->n)) == NULL) return NULL;
  
  N = 2 * N + 1;
  
  t = 2.0 * M_PI / N;
  
  dft_r2r_1d(p->n, p->N, p->X, h, 0.0);                 /* Compute h(t) */
  (p->f)(p->n, p->n, h, h + p->n, NULL, p->w, 0.0);     /* Compute f(h(t)) */
  dft_r2r_1d(p->n, p->N, p->X, h, p->w);                /* Compute h(t+w) */
  
  error = fabs(h[0] - h[p->n]); k = 0;                  /* Compute max |f(h(t)) - h(t+w)| */
  for (j = 1; j < p->n; j ++) {
    aux = fabs(h[j] - h[j + p->n]);
    if (error < aux) error = aux;
  } 
  
  for (i = 1; i < N; i ++) {
    dft_r2r_1d(p->n, p->N, p->X, h, t * i);
    (p->f)(p->n, p->n, h, h + p->n, NULL, p->w, t * i);
    dft_r2r_1d(p->n, p->N, p->X, h, t * i + p->w);
    
    for (j = 0; j < p->n; j ++) {
      aux = fabs(h[j] - h[j + p->n]);
      if (error < aux) { error = aux; k = i; }
    }
  }
  
  free(h);  
  if (max) *max = error;        /* Maximum value */
  if (idx) *idx = k;            /* Index where the maximum value is reached */
  
  return (void *) 1;
}

/*!
 * \fn void pic_x(size_t n, size_t N, double * const X, double * const x)
 * \brief Function that compute the Discrete Fourier Transform
 * 
 * \param[in] n Space's dimension
 * \param[in] N Mesh size
 * \param[in] X Fourier coefficients, that is, n(2N+1) components
 * \param[in] x Discrete Fourier Transform for each mesh's element, that is, n(2N+1) components
 * 
 * \pre The pointers are not NULL
 * \post The vector x has been changed
 */
void pic_x(size_t n, size_t N, double * const X, double * const x) {
  if (X == NULL || x == NULL) return;
  
  size_t m, i;
  double t;
  
  m = 2 * N + 1;
  t = 2 * M_PI / m;
  
  for (i = 0; i < m; i ++) 
    dft_r2r_1d(n, N, X, x + i * n, t * i);
}

/*
 * \fn size_t _pic_estimate_mesh_size(struct pic * const p, double tol, double ** A, double * h)
 * \brief Function that estimates the mesh size for the Discrete Fourier Transform
 * 
 * \param[in] p Pointer to struct pic
 * \param[in] tol tolerance
 * \param[in] A Auxiliar matrix, can be NULL
 * \param[in] h Auxiliar vector, can be NULL
 * 
 * \pre The struct pointer is not NULL
 * 
 * \return New candidate mesh size
 */
size_t _pic_estimate_mesh_size(struct pic * const p, 
                               double tol, 
                               double ** A, 
                               double * h) {
  size_t m, n, N, rows, cols, max;
  size_t i, j;
  double *a, *b, d;
  
  short unsigned int flag_vector = 0, flag_matrix = 0;
  
  switch (p->N) {
    case 0:
      return 0; /* Should not be possible */
    case 1:
      rows = 1;
      break;
    case 2:
      rows = 3;
      break;
    default:
      rows = 4;
      break;
  }
  
  cols = 2;
  
  if (A == NULL) {
    if ((A = malloc_matrix(rows, cols)) == NULL) return 0;
    flag_matrix = 1;
  }
  if (h == NULL) {
    if ((h = malloc_vector(rows)) == NULL) {
      if (flag_matrix == 1)
        free_matrix(rows, A);
      return 0;
    }
    flag_vector = 1;
  }
  
  n = p->n;
  m = n * (2 * p->N + 1);
  max = p->N;
  
//   N = p->N;
//   b = (p->X) + m - 1;
//   a = b - n;
//   
//   for (j = 0; j < rows; j ++) {
//     for (i = 0; i < n; i ++) {
//       
//     }
//   }
  for (i = 0; i < n; i ++) {
    N = p->N;
    b = (p->X) + m - 1 - i;
    a = b - n;
    
    for (j = 0; j < rows; j ++) {          
        d = sqrt( (*a) * (*a) + (*b) * (*b) );
        
        #ifdef _PIC_DEBUG2
        if (d == 0.0)
          printf("%zu-th Fourier coefficient in %zu-th coordinate\n", N, i);
        #endif
        
        A[j][0] = 1.;
        A[j][1] = -N;
        
        h[j] = log(d);
        N --;
        
        a -= n; b -= n;
    }
    
    //     if (solve_qr(rows, 2, A, h, h, tol) == NULL) {
    if (qrsis(rows, 2, A, h, tol) == 0) {
        return 0;
    }
    #ifdef _PIC_DEBUG2
    printf("Solve for rows = %zu with solution ", j);
    print_vector(0, rows, h, "x");
    #endif
    
    d = fabs( ( log(tol) - h[1] ) / h[0] );
    N = d; // Cast
    
    #ifdef _PIC_DEBUG2
    printf("N = %f and N = %zu\n", d, N);
    #endif
    if (N > max) max = N;
  }
  
//   printf("a = %24.16e \t b = %24.16e\n", *a, *b);
  
  if (flag_vector == 1)
    free_vector(h);
  
  if (flag_matrix == 1)
    free_matrix(rows, A);
  
  return max;
  
//   for (i = 0; i < n; i ++) {
//     for (j = 0; j < rows; j ++) {
//       k = 0;
//       
//       a = (p->X)[m - 1 - (k+1)*n - i - j];
//       b = (p->X)[m - 1 - k*n - i - j];
//       
//       a = sqrt(a*a + b*b);
//       
//       #ifdef _PIC_DEBUG
//       if (a <= 0.0)
//         printf("%zu-th Fourier coefficient in %zu-th coordinate\n", k, i);
//       #endif
//         
//       A[j][0] = 1.;
//       A[j][1] = -N;
//       
//       h[j] = log(a);
//       
//       k += 2;
//       N --;
//     }
//     
//     //     if (solve_qr(rows, 2, A, h, h, tol) == NULL) {
//     if (qrsis(rows, 2, A, h, tol) == 0) {
//       return 0;
//     }
//     
//     #ifdef _PIC_DEBUG
//     printf("Solve for n = %zu with solution ", j);
//     print_vector(0, rows, h, "x");
//     #endif
//     
//     a = fabs( ( log(tol) - h[1] ) / h[0] );
//     N = a;
//     
//     #ifdef _PIC_DEBUG
//     printf("N = %f and N = %zu\n", a, N);
//     #endif
//     if (N > max) max = N;
//     
//     N = p->N;
//   }
//   
//   if (flag_vector == 1)
//     free_vector(h);
//   
//   if (flag_matrix == 1)
//     free_matrix(rows, A);
//   
//   return max;
//   // despres
//     
//   for (j = 0; j < n; j ++) {
//     k = 0;
//     for (i = 0; i < rows; i ++) {
// //       a = MAX( fabs( (p->X)[m - 1 - k*n - j] ), fabs( (p->X)[m - 1 - (k+1)*n - j] ) );
//       a = (p->X)[(k + 1)*n + j];
//       b = (p->X)[k*n + j];
//       
//       a = sqrt( a*a + b*b );
//       
//       (p->x)[i] = log(a);
//       (p->DF)[i][0] = 1.0;
//       (p->DF)[i][1] = -i; //ABS(p->N - i);
//       
//       k += 2;
//     }
//     
// #ifdef _PIC_DEBUG
//     printf("\nIndependent terms for n = %zu with solution ", j);
//     print_vector(0, rows, p->x, "x");
// #endif
//     
// //     if (solve_qr(rows, 2, p->DF, p->x, p->x, tol) == NULL)
//     if (qrsis(rows, 2, p->DF, p->x, tol) == 0)
//       return 0;
//     
// #ifdef _PIC_DEBUG
//       printf("Solve for n = %zu with solution ", j);
//       print_vector(0, rows, p->x, "x");
// #endif
//       a = fabs( ( log(tol) - (p->x)[0] ) / (p->x)[1] );
//       
// #ifdef _PIC_DEBUG
//       printf("N = %f and N = %zu\n", a, (size_t) ceil(a));
// #endif
//       if (a > N) N = (size_t) ceil(a);
//   }
//   return N;
}

size_t pic_estimate_mesh_size(struct pic * const p, double tol) {
  return _pic_estimate_mesh_size(p, tol, NULL, NULL);
}

/*!
 * \fn void * pic_execute(struct pic * const p, double tol)
 * \brief Function that compute the Discrete Fourier Transform
 * 
 * \param[in] p Pointer to the pic struct
 * \param[in] tol Tolerance for the result
 * 
 * \return NULL if some problem ocurrer. Otherwise the number of iterations performed until the result (minimum 1)
 * 
 * \note The mesh size of pic struct can have been changed. Use \see pic_get_mesh_size
 * 
 * \pre The pointers are not NULL
 * \post The vector p->x has the solution
 */
void * pic_execute(struct pic * const p, double tol) {
  
  if (_init_pic(p) == NULL) return NULL;
  
  size_t k, idx;
  double max;
  
  for (k = 1; k < max_iterations+1; k ++) {
    if (newton_execute(p->n, p->N, p->x, p->X, p->DF, p->w, tol, p->f, p->solve, p->autonomous) == NULL) {
      #ifdef _PIC_DEBUG
      printf("Can not compute newton_execute in %zu-th iteration\n", k);
      #endif
      
      return NULL;
    }
    
    if (pic_error(p, 4 * p->N, &idx, &max) == NULL) {
      #ifdef _PIC_DEBUG
      printf("Can not compute pic_error with mesh size %zu in %zu-th iteration\n", 4 * p->N, k);
      #endif
      
      return NULL;
    }
    
    #ifdef _PIC_DEBUG
    printf("Calculus error: %e in index %zu\n", max, idx);
    #endif
    
    if (max <= tol)
      break;
    
    #ifdef _PIC_DEBUG
    printf("Set mesh size from %zu ", p->N);
//     printf("\nNew mesh size = %zu\n", idx);
    #endif
    
    idx = pic_estimate_mesh_size(p, tol);
    if (idx == 0 || idx > 10000 || idx == p->N)
      pic_set_mesh_size(p, p->N + 2);
    else
      pic_set_mesh_size(p, idx);
      
//     break;
//     pic_set_mesh_size(p, pic_estimate_mesh_size(p, tol));
//     pic_set_mesh_size(p, p->N + 2);
    
    #ifdef _PIC_DEBUG
    printf("to %zu\n", p->N);
    #endif
  }
  
  pic_x(p->n, p->N, p->X, p->x);
  
  return (void *) k;  
}

/*!
 * \fn double ** pic_differential(struct pic * const p, size_t * const m, size_t * const n)
 * \brief Return a differential matrix for the current pic struct's values
 * 
 * \param[in] p Pointer to a pic struct
 * \param[out] m Number of differential matrix's rows
 * \param[out] n Number of differential matrix's columns
 * 
 * \return Pointer to initial position of the matrix differential or NULL if some problem occurred
 * 
 * \pre The pointer to a pic struct is not NULL
 * \note The pointers for rows and columns can be NULL
 */
double ** pic_differential(struct pic * const p, size_t * const m, size_t * const n) {
  if (p == NULL || p->DF == NULL) return NULL;
  
  if (m) *m = p->n * (2 * p->N + 1) + p->autonomous;
  if (n) *n = p->n * (2 * p->N + 1);
  
  newton_differential(p->n, p->N, p->x, p->X, p->DF, p->f, p->w, NULL);
  
  return p->DF;
}

/*!
 * \fn size_t pic_get_n(struct pic * const p)
 * 
 * \param[in] p Pointer to a pic struct
 * 
 * \return Space's dimension of the pic struct
 * 
 * \pre The pointer is not NULL
 */
size_t pic_get_n(struct pic * const p) {
  return p->n;
}

/*!
 * \fn size_t pic_get_N(struct pic * const p)
 * 
 * \param[in] p Pointer to a pic struct
 * 
 * \return The mesh size of the pic struct
 * 
 * \pre The pointer is not NULL
 */
size_t pic_get_N(struct pic * const p) {
  return p->N;
}

/*!
 * \fn double * pic_get_x(struct pic * const p)
 * 
 * \param[in] p Pointer to a pic struct
 * 
 * \return The value table of the curve solution of the pic struct
 * 
 * \pre The pointer is not NULL
 */
double * pic_get_x(struct pic * const p) {
  return p->x;
}

/*!
 * \fn double * pic_get_X(struct pic * const p) 
 * 
 * \param[in] p Pointer to a pic struct
 * 
 * \return The Fourier's coefficients of the pic struct
 * 
 * \pre The pointer is not NULL
 */
double * pic_get_X(struct pic * const p) {
  return p->X;
}

/*!
 * \fn double ** pic_get_DF(struct pic * const p)
 * 
 * \param[in] p Pointer to a pic struct
 * 
 * \return The pointer to the differential of the pic struct
 * 
 * \pre The pointer is not NULL
 */
double ** pic_get_DF(struct pic * const p) {
  return p->DF;
}

/*!
 * \fn double pic_get_w(struct pic * const p)
 * 
 * \param[in] p Pointer to a pic struct
 * 
 * \return The real number of the pic struct
 * 
 * \pre The pointer is not NULL
 */
double pic_get_w(struct pic * const p) {
  return p->w;
}