/*! 
 * \file save.c
 * \brief Implements functions that save a vector or a matrix in some file
 *
 * \author Joan Gimeno
 * \date 22/05/2014 (start)
 * \version 1.0
 */
#include "save.h"

#define EPS 1e-10

/*!
 * \fn int save_vector(size_t idx, size_t n, double * const b, FILE * const f)
 * \brief Function that saves the values of a vector in a given file
 * 
 * \param[in] idx Start index
 * \param[in] n Dimension
 * \param[in] b Pointer to a vector
 * \param[in] f Output file
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int save_vector(size_t idx, size_t n, 
                double * const b, FILE * const f) {
  if (idx >= n) return 0;
  
  typeof(n) i;
  double *p = b + idx;
  
  for (i = idx; i < n; i ++, p ++)
    if (fprintf(f, "%24.16e\n", *p) < 0)
      return 0;
    
  return 1;
}

/*!
 * \fn int saveas_vector(size_t idx, size_t n, double * const b, char * const name)
 * \brief Function that saves the values of a vector in a given filename
 * 
 * \param[in] idx Start index
 * \param[in] n Dimension
 * \param[in] b Pointer to a vector
 * \param[in] name Filename
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int saveas_vector(size_t idx, size_t n, 
                  double * const b, char * const name) {
  if (idx >= n) return 0;
  
  int r;
  FILE *f;
  
  if ((f = fopen(name, "w")) == NULL)
    return 0;
  
  r = save_vector(idx, n, b, f);
    
  return (fclose(f) != EOF) && (r == 1);
}

/*!
 * \fn int save_vector_indexs(size_t idx, size_t n, double * const b, FILE * const f)
 * \brief Function that saves the values not zero of a vector with its indexs in a given file
 * 
 * \param[in] idx Start index
 * \param[in] n Dimension
 * \param[in] b Pointer to a vector
 * \param[in] f Output file
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int save_vector_indexs(size_t idx, size_t n, 
                       double * const b, FILE * const f) {
  if (idx >= n) return 0;
  
  typeof(n) i;
  double *p = b + idx;
  
  for (i = idx; i < n; i ++, p ++)
    if (fabs(*p) >= EPS)
      if (fprintf(f, "%zu %24.16e\n", i, *p) < 0)
        return 0;
    
  return 1;
}

/*!
 * \fn int saveas_vector_indexs(size_t idx, size_t n, double * const b, char * const name)
 * \brief Function that saves the values not zero of a vector with its indexs in a given filename
 * 
 * \param[in] idx Start index
 * \param[in] n Dimension
 * \param[in] b Pointer to a vector
 * \param[in] name Filename
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int saveas_vector_indexs(size_t idx, size_t n, 
                         double * const b, char * const name) {
  if (idx >= n) return 0;
  
  int r;
  FILE *f;
  
  if ((f = fopen(name, "w")) == NULL)
    return 0;
  
  r = save_vector_indexs(idx, n, b, f);
    
  return (fclose(f) != EOF) && (r == 1);
}

/*!
 * \fn int save_matrix(size_t idx_row, size_t m, size_t idx_col, size_t n, double ** const A, FILE * const f)
 * \brief Function that saves the values of a matrix in a given file
 * 
 * \param[in] idx_row Start index row
 * \param[in] m Rows
 * \param[in] idx_col Start index column
 * \param[in] n Columns
 * \param[in] A Pointer to a matrix
 * \param[in] f Output file
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int save_matrix(size_t idx_row, size_t m, 
                size_t idx_col, size_t n, 
                double ** const A, FILE * const f) {
  if (idx_row >= m || idx_col >= n) 
    return 0;
  
  typeof(n) i, j;
  
  for (i = idx_row; i < m; i ++) {
    
    for (j = idx_col; j < n; j ++)
      if (fprintf(f, "%24.16e ", A[i][j]) < 0)
        return 0;
      
    if (fprintf(f, "\n") < 0)
      return 0;
  }
      
  return 1;
}

/*!
 * \fn int saveas_matrix(size_t idx_row, size_t m, size_t idx_col, size_t n, double ** const A, char * const name)
 * \brief Function that saves the values of a matrix in a given filename
 * 
 * \param[in] idx_row Start index row
 * \param[in] m Rows
 * \param[in] idx_col Start index column
 * \param[in] n Columns
 * \param[in] A Pointer to a matrix
 * \param[in] name Filename
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int saveas_matrix(size_t idx_row, size_t m, 
                  size_t idx_col, size_t n, 
                  double ** const A, char * const name) {
  if (idx_row >= m || idx_col >= n) 
    return 0;
  
  int r;
  FILE *f;
  
  if ((f = fopen(name, "w")) == NULL)
    return 0;
  
  r = save_matrix(idx_row, m, idx_col, n, A, f);
  
  return (fclose(f) != EOF) && (r == 1);
}

/*!
 * \fn int save_matrix_indexs(size_t idx_row, size_t m, size_t idx_col, size_t n, double ** const A, FILE * const f)
 * \brief Function that saves the values not zero of a matrix with its indexs in a given file
 * 
 * \param[in] idx_row Start index row
 * \param[in] m Rows
 * \param[in] idx_col Start index column
 * \param[in] n Columns
 * \param[in] A Pointer to a matrix
 * \param[in] f Output file
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int save_matrix_indexs(size_t idx_row, size_t m, 
                       size_t idx_col, size_t n, 
                       double ** const A, FILE * const f) {
  if (idx_row >= m || idx_col >= n) 
    return 0;
  
  typeof(n) i, j;
  
  for (i = idx_row; i < m; i ++) {
    
    for (j = idx_col; j < n; j ++) {
      if (fabs(A[i][j]) >= EPS)
        if (fprintf(f, "%zu %zu %24.16e\n", i, j, A[i][j]) < 0)
          return 0;
    }
  }
      
  return 1;
}

/*!
 * \fn int saveas_matrix_indexs(size_t idx_row, size_t m, size_t idx_col, size_t n, double ** const A, char * const name)
 * \brief Function that saves the values not zero of a matrix with its indexs in a given filename
 * 
 * \param[in] idx_row Start index row
 * \param[in] m Rows
 * \param[in] idx_col Start index column
 * \param[in] n Columns
 * \param[in] A Pointer to a matrix
 * \param[in] name Filename
 * 
 * \retval 0 If some error has occurred in some moment of the process
 * \retval 1 Otherwise
 * 
 * \pre The pointers are not NULL
 * \post The file with name has been created or modified with the result
 */
int saveas_matrix_indexs(size_t idx_row, size_t m, 
                         size_t idx_col, size_t n, 
                         double ** const A, char * const name) {
  if (idx_row >= m || idx_col >= n) 
    return 0;
  
  int r;
  FILE *f;
  
  if ((f = fopen(name, "w")) == NULL)
    return 0;
  
  r = save_matrix_indexs(idx_row, m, idx_col, n, A, f);
  
  return (fclose(f) != EOF) && (r == 1);
}

// #include <stdlib.h>
// int main(int argc, char **argv) {
//   double **A, *b;
//   int i, j, m, n;
//   m = n = 10;
//   
//   if ((b = (double *) malloc(m*sizeof(double))) == NULL)
//     return 1;
//   
//   if ((A = (double **) malloc(m*sizeof(double *))) == NULL) {
//     free(b);
//     return 1;
//   }
//   
//   for (i = 0; i < m; i ++)
//     if ((A[i] = (double *) malloc(n*sizeof(double))) == NULL) {
//       for (j = 0; j < i; j ++)
//         free(A[i]);
//       free(A); free(b);
//       return 1;
//     }
//       
//   time_t t;
//   srand((unsigned) time(&t));
//   
//   for (i = 0; i < m; i ++) {
//     b[i] = rand() % 100;
//     for (j = 0; j < n; j ++)
//       A[i][j] = rand() % 100;
//   }
//     
//   saveas_matrix(0, m, 0, n, A, "A.txt");
//   saveas_matrix_indexs(0, m, 0, n, A, "test_matrix.txt");
//   saveas_vector_indexs(0, n, b, "test_vector.txt");
//   
//   free(b);
//   
//   for (i = 0; i < m; i ++)
//     free(A[i]);
//   free(A);
//     
//   return 0;
// }