/*!
 * \mainpage Approximations of invariant curves of diffeomorphisms
 * 
 * \author Joan Gimeno
 * \date Spring 2014
 * 
 * \section Objectives What is the utility of the current library ?
 * 
 * Assumes that we have a discret system like that:
 * \f[
 * \begin{cases}
 * \bar x = f(x, \theta) + \varepsilon g(x,\theta)\\
 * \bar \theta = \theta + \omega,
 * \end{cases}
 * \f]
 * where \f$ f, g \f$ are differential maps with the second variable on the torus.
 * 
 * A continuous map \f$ x\colon \mathbb{T} \rightarrow \mathbb{R} \f$ is an invariant curve of the discret system if, and only if, 
 * \f[
 * x(\theta + \omega) = f(x(\theta), \theta) + \varepsilon g(x(\theta),\theta) \quad \text{ for all } \theta.
 * \f]
 * The value \f$ \omega \f$  is known as the rotation number of the curve and it is common to suppose that is a irrational real number.
 * 
 * The library compute an approximation of some invariant curve of the discret system.
 */
/*!
 * \file utils.c
 * \brief Set of different function for linear systems 
 * 
 * \author Joan
 * \date 04/12/2013 (start)
 */

#include "utils.h"

#ifdef DEBUG
#define _MAX_STRING_LENGTH 50
#endif

/*!
 * \fn double euclidian_norm(size_t n, double * const x)
 * \brief Derive the euclidian norm of a vector, that is, \f$ \displaystyle || x ||_2 = \biggl(\sum_{i = 0}^{n-1} x_i^2\biggr)^{\frac 12} \f$
 * 
 * \param[in] n Dimension of vector
 * \param[in] x Input vector
 * 
 * \return Euclidian norm
 * 
 * \post Thre result is greater than or equal to zero
 * \note If the pointer is NULL, the result is zero
 */
double euclidian_norm(size_t n, double * const x) {
  if (x == NULL) return 0;
  
  typeof(n) i;
  typeof(*x) aux = 0.0;
  
  for (i = 0; i < n; i ++)
    aux += x[i] * x[i];
  
  assert(aux >= 0.0);
  
  return sqrt(aux);
}

/*!
 * \fn double inner_product(size_t n, double * const x)
 * \brief Derive the ordinary inner product of a vector, that is, \f$ \displaystyle \langle x, x \rangle = \sum_{i = 0}^{n-1} x_i^2 \f$
 * 
 * \param[in] n Dimension of vector
 * \param[in] x Input vector
 * \return Inner product
 * 
 * \post The result is greater than or equal to zero
 * \note If the pointer is NULL, the result is zero
 */
double inner_product(size_t n, double * const x) {
  if (x == NULL) return 0;
  
  typeof(n) i;
  typeof(*x) aux = 0.0;
  for (i = 0; i < n; i ++)
    aux += x[i] * x[i];
  
  assert(aux >= 0.0);
  
  return aux;
}

/*!
 * \fn void copy_vector(size_t n, double * const x, double * const y)
 * \brief Copy the vector x in y
 *
 * \param[in] n Dimension
 * \param[in] x Source vector
 * \param[out] y Destination vector
 * 
 * \pre The pointers are not NULL
 * \warning The pointers should not be the same
 */
void copy_vector(size_t n, double * const x, double * const y) {
  typeof(n) i;
  
  #pragma omp parallel if (n > _OPENMP_DIMENSION) default(shared) private(i)
  {
    #pragma omp for
    for (i = 0; i < n; i ++)
      y[i] = x[i];
  }
}

/*!
 * \fn void copy_matrix(size_t m, size_t n, double ** const A, double ** const B)
 * \brief Copy the matrix A in B
 *
 * \param[in] m Rows
 * \param[in] m Columns
 * \param[in] A Source matrix
 * \param[out] B Destination matrix
 * 
 * \pre The pointers are not NULL
 * \warning The pointers should not be the same
 */
void copy_matrix(size_t m, size_t n, double ** const A, double ** const B) {
  typeof(n) i, j;
  
  #pragma omp parallel if (n > _OPENMP_DIMENSION) default(shared) private(i, j)
  {
    #pragma omp for
    for (i = 0; i < m; i ++)
      for (j = 0; j < n; j ++)
        B[i][j] = A[i][j];
  }
}

/*!
 * \fn void print_matrix(size_t m, size_t n, double ** const A, char * const name)
 * \brief Display in stdout a matrix m-by-n
 *
 * \param[in] m Rows
 * \param[in] n Columns
 * \param[in] A Matrix m-by-n
 * \param[in] name Name of the matrix in stdout
 * 
 * \pre The dimension are not zeros and the pointers are not NULL
 */
void print_matrix(size_t m, size_t n, double ** const A, char * const name) {
  printf("%s = \n", name);
  
  size_t i, j;
  for (i = 0; i < m; i ++) {
    for (j = 0; j < n; j ++)
      printf("%f ", A[i][j]);
    printf("\n");
  }
  printf("\n");
}

/*!
 * \fn void print_vector(size_t idx, size_t n, double * const v, char * const name)
 * \brief Display in stdout a n-dimensional vector starting by an index
 *
 * \param[in] idx Start index
 * \param[in] n End index
 * \param[in] v Vector
 * \param[in] name Name of the vector in stdout
 * 
 * \pre The idx is less than n and the pointers are not NULL
 */
void print_vector(size_t idx, size_t n, double * const v, char * const name) {
  printf("%s = \n", name);
  
  size_t i;
  for (i = idx; i < n; i ++)
    printf("%f ", v[i]);
    
  printf("\n");
}

/*!
 * \fn double * malloc_vector(size_t n)
 * \brief Malloc a n-dimensional vector
 *
 * \param[in] n End index
 * 
 * \return Pointer to a malloc vector
 * 
 * \note The return value could be NULL
 */
double * malloc_vector(size_t n) {
  #ifdef DEBUG
  char str[_MAX_STRING_LENGTH];
  #endif
  double *p;
  
  if ((p = (double *) malloc(n * sizeof(double))) == NULL) {
    #ifdef DEBUG
    snprintf(str, _MAX_STRING_LENGTH, "Unsuccess allocated memory for a %zu-dimensional vector", n);
    perror(str);
    #endif
    return NULL;
  }
  return p;
}

/*!
 * \fn void free_vector(double *p)
 * \brief Free a vector
 *
 * \param[in] p Pointer
 */
void free_vector(double *p) {
  free(p); p = NULL;
}

/*!
 * \fn double ** malloc_matrix(size_t m, size_t n)
 * \brief Malloc a m-by-n matrix
 *
 * \param[in] m Rows
 * \param[in] n Columns
 * 
 * \return Pointer to a matrix
 * 
 * \note The return value could be NULL
 */
double ** malloc_matrix(size_t m, size_t n) {
  #ifdef DEBUG
  char str[_MAX_STRING_LENGTH];
  #endif
  
  double **p;
  typeof(m) i, j;
  
  if ((p = (double **) malloc(m * sizeof(double *))) == NULL) {
    #ifdef DEBUG
    snprintf(str, _MAX_STRING_LENGTH,"Unsuccess allocated memory for a %zu-by-%zu maxtrix", m, n);
    perror(str);
    #endif
    return NULL;
  }
  
  for (i = 0; i < m; i ++) {
    if ((p[i] = (double *) malloc(n * sizeof(double))) == NULL) {
      for (j = 0; j < i; j ++) {
        free(p[j]); p[j] = NULL;
      }
      free(p); p = NULL;
      
      #ifdef DEBUG
      snprintf(str, _MAX_STRING_LENGTH, "Unsuccess allocated memory for a %zu-by-%zu maxtrix in row %zu", m, n, i);
      perror(str);
      #endif
      
      return NULL;
    }
  }
  
  return p;
}

/*!
 * \fn void free_matrix(size_t m, double **p)
 * \brief Free a matrix with m rows
 *
 * \param[in] m Rows
 */
void free_matrix(size_t m, double **p) {
  if (p == NULL) return;
  
  typeof(m) i;
  for (i = 0; i < m; i ++) {
    free(p[i]); p[i] = NULL;
  }
  
  free(p); p = NULL;
}

#ifdef _MEMORY_INFO
void add_bytes(size_t b) {
  m.used += b;
}

void print(void) {
  printf("\tMemory used %zu\n\tMemomry free %zu\n\tTotal", m.used, m.free, m.used + m.free);  
}

void free_bytes(size_t b) {
  if (m.used < b) {
    printf("Impossible free %zu bytes\n", b);
  } else {
    m.used -= b;
    m.free += b;
  }
}
#endif

void allocated_vector(void *p, size_t n) {  
  p = malloc(n);
  #ifdef _MEMORY_INFO
  if (p != NULL) add_bytes(n);
  #endif
}

void free_vector2(void *p, size_t n) {
  free(p);
  #ifdef _MEMORY_INFO
  free_bytes(n);
  #endif
}

void allocated_matrix(void ** p, size_t m, size_t n) {
  typeof(n) i, j;
  
  if ((p = malloc(m)) == NULL) return;
  
  #ifdef _MEMORY_INFO
  add_bytes(m);
  #endif
  
  for (i = 0; i < m; i ++) {
    if ((p[i] = malloc(n)) == NULL) {
      for (j = 0; j < i; j ++) free_vector2(p[j], n);
      free(p);
      p = NULL;
      
      #ifdef _MEMORY_INFO
      free_bytes(m);
      #endif
      
      break;
    }
    
    #ifdef _MEMORY_INFO
    add_bytes(n);
    #endif
  }
}

void free_matrix2(void **p, size_t m, size_t n) {
  typeof(n) i;
  
  for (i = 0; i < m; i ++) free_vector2(p[i], n);
  free(p);
  
  #ifdef _MEMORY_INFO
  free_bytes(m);
  #endif
}