#include "pic.h"
#include "taylor-pendol-diff.h"
#include "taylor-pendol.h"
#include "paraview.h"
#include <getopt.h>
#include <string.h>

#define MAX_LENGTH_STRING 32
#define DEFAULT_TOLERANCE 1e-12
#define DEFAULT_MESH_SIZE_1 45
#define DEFAULT_MESH_SIZE_2 50
#define DEFAULT_EPSILON 1e-2
#define DEFAULT_MU 0.

MY_FLOAT epsilon, mu;

double w1, w2;

void compute_taylor_with_differential(size_t n, double * const x, double * const init, MY_FLOAT endtime, MY_FLOAT theta) {
  /* Initial conditions */
  typeof(n) i;
  for (i = 0; i < n; i ++)
    init[i] = x[i];
  init[i] = theta;
  
  /* Initial conditions of the differential */
  i ++;
  init[i] = 1.;
  i ++;
  init[i] = 0.;
  i ++;
  init[i] = 0.;
  i ++;
  init[i] = 1.;
  
  int order = 10;
  MY_FLOAT t = 0., h_return = 10;
  
  while ( taylor_step_pendol_diff(&t, init, 1, 2, -16, -16, &endtime, &h_return, &order) != 1 );
}

void compute_taylor(size_t n, double * const x, double * const init, MY_FLOAT endtime, MY_FLOAT theta) {
  /* Initial conditions */
  double a[n+1];
  typeof(n) i;
  for (i = 0; i < n; i ++)
    a[i] = x[i];
  a[i] = theta;
  
  int order = 10;
  MY_FLOAT t = 0., h_return = 10;
  
  while ( taylor_step_pendol(&t, a, 1, 2, -16, -16, &endtime, &h_return, &order) != 1 );
  
  for (i = 0; i < n; i ++)
    init[i] = a[i];
}

void f(size_t m, size_t n, double * const x, double * const f, double ** const df, double w, double theta) {
  double a[m*n + n + 1];
  
  compute_taylor_with_differential(n, x, a, 2 * M_PI / w2, theta);
  
//   size_t i;
//   for (i = 0; i < n; i ++)
//     f[i] = a[i];
  f[0] = a[0];
  f[1] = a[1];
  
  if (df == NULL) return;
  
  df[0][0] = a[3];
  df[0][1] = a[4];
  df[1][0] = a[5];
  df[1][1] = a[6];
  
//   size_t j;
//   for (i = 0; i < m; i ++)
//     for (j = 0; j < n; j ++)
//       df[i][j] = a[(i + 1) * m + j];
}

int flow(size_t dim, size_t M, double * const init, char * const name) {
  size_t i, m;
  double *v, *p, hi, ti;
  
  m = 2 * M + 1;
  hi = 2 * M_PI / m;
  
  m ++;
  if ((v = malloc_vector(dim * m)) == NULL)
    return 0;
  
  p = v;  
  for (i = 0; i < m; i ++, p += dim) {
    ti = i * hi / w2;
    compute_taylor(dim, init, p, ti, 0.0);
  }
  
  if (saveas_flow(dim, 1, m, v, name) == 0) {
    free_vector(v);
    return 0;
  }
  
  free_vector(v);
  return 1;
}
         
int surface(size_t dim,
            size_t mesh_size_1,
            size_t mesh_size_2,
            double * const dft,
            double w1, 
            double w2,
            char * const name) {
  size_t i, j, m, n;
  double ti, tj, hi, hj, *v, *p;
  
  n = 2 * mesh_size_1 + 1;
  m = 2 * mesh_size_2 + 1;
  
  hi = 2 * M_PI / n;
  hj = 2 * M_PI / m;
  
  n ++; m ++;
  if ((v = malloc_vector(dim * n * m)) == NULL)
    return 0;
  
  p = v;
  for (j = 0; j < m; j ++) {
    tj = j * hj / w2;
    
    for (i = 0; i < n; i ++, p += dim) {
      ti = i * hi - w1 * tj;
      
      dft_r2r_1d(dim, mesh_size_1, dft, p, ti);
      compute_taylor(dim, p, p, tj, ti);
    }
  }
  
  saveas_surface(dim, n, m, v, name);
  
  flow(dim, mesh_size_2, v, "data/flow.vtk");
//   flow(dim, mesh_size_2, v + dim, "data/flow1.vtk");
  
  free_vector(v);
  return 1;
}

void print_usage(char * const name) {
  printf("Usage: %s \n\t-h, --help\t\t\tHelp instruction\n\t-n, --dim value\t\t\tDimension\n\t-N, --mesh-size-1 value\t\tCurve's mesh size\n\t-M, --mesh-size-2 value\t\tSurface's mesh size\n\t-l, --lu\t\t\tSolve the linear system of equations with LU descomposition\n\t-q, --qr\t\t\tSolve the linear system of equations with QR descomposition\n\t-t, --tolerance value\t\tTolerance of the numeric method (default: %1.2e)\n\t-e, --epsilon value\t\tExtern auxiliar variable (default: %1.2e)\n\t-m, --mu value\t\t\tFriction coefficient (default: %1.2e)\n\t--w1 value\t\t\tFrequence 1\n\t--w2 value\t\t\tFrequence 2\n\t--flag\t\t\t\tCompute the flow and surface with the output-fourier\n\t-p, --proc value\t\tNumber of threads that will be used\n\t-s, --input-fourier name\tInput file name with Fourier's coefficients\n\t-o, --output name\t\tOutput file name with the solution\n\t--output-fourier name\t\tOutput file name with Fourier's coefficients\n\t--output-matrix name\t\tOutput file name with the differential\n\t--output-flow name\t\tOutput file name with the flow\n\t--output-surface name\t\tOutput file name with the surface\n", name, DEFAULT_TOLERANCE, DEFAULT_EPSILON, DEFAULT_MU);
}

int main(int argc, char **argv) {
  double w, tol;
  size_t n, N, M;
  short int lu = -1, qr = 1, flag = -1;
  
  struct pic *p;
  
  /* Default values */
  epsilon = DEFAULT_EPSILON;
  mu = DEFAULT_MU;
  
  tol = DEFAULT_TOLERANCE;
  n = 2;
  N = DEFAULT_MESH_SIZE_1;
  M = DEFAULT_MESH_SIZE_2;
  w1 = sqrt(3); w2 = M_SQRT2;
  
  char input_fourier[MAX_LENGTH_STRING]  = "pendol_init_fourier.dat";
  char output[MAX_LENGTH_STRING]         = "data/test4.dat";
  char output_fourier[MAX_LENGTH_STRING] = "data/test4_coef_fourier.dat";
  char output_matrix[MAX_LENGTH_STRING]  = "data/test4_differential.dat";
  char output_flow[MAX_LENGTH_STRING]    = "data/flow.vtk";
  char output_surface[MAX_LENGTH_STRING] = "data/surface.vtk";
  
  static struct option long_options[] = {
    {"help",                    no_argument,            0,      'h'},
    {"flag",                    no_argument,            0,      1},
    {"lu",                      no_argument,            0,      'l'},
    {"qr",                      no_argument,            0,      'q'},
    {"dim",                     required_argument,      0,      'n'},
    {"mesh-size-1",             required_argument,      0,      'N'},
    {"mesh-size-2",             required_argument,      0,      'M'},
    {"tolerance",               required_argument,      0,      't'},
    {"epsilon",                 required_argument,      0,      'e'},
    {"mu",                      required_argument,      0,      'm'},
    {"w1",                      required_argument,      0,      2},
    {"w2",                      required_argument,      0,      3},
    {"proc",                    required_argument,      0,      'p'},
    {"input-fourier",           required_argument,      0,      's'},
    {"output",                  required_argument,      0,      'o'},
    {"output-fourier",          required_argument,      0,      4},
    {"output-matrix",           required_argument,      0,      5},
    {"output-flow",             required_argument,      0,      6},
    {"output-surface",          required_argument,      0,      7},
    {NULL,                      0,                      NULL,   0}
  };
  int long_index = 0;
  
//   #if defined (_OPENMP)
//   omp_set_num_threads(1);
//   #endif
  
  /* Evaluate arguments */
  int c;
  while ((c = getopt_long(argc, argv, "hM:N:m:n:lqt:w:e:s:o:p:", long_options, &long_index)) != EOF) {
    switch (c) {
      case 1:                   /* Flag */
        flag = 1;
        break;
      case 2:                   /* w1 */
        w1 = atof(optarg);
        break;
      case 3:                   /* w2 */
        w2 = atof(optarg);
        break;
      case 4:                   /* Name output Fourier coefficients file */
        strncpy(output_fourier, optarg, MAX_LENGTH_STRING);
        break;
      case 5:                   /* Name output matrix file */
        strncpy(output_matrix, optarg, MAX_LENGTH_STRING);
        break;
      case 6:                   /* Name output curve file */
        strncpy(output_flow, optarg, MAX_LENGTH_STRING);
        break;
      case 7:                   /* Name output surface file */
        strncpy(output_surface, optarg, MAX_LENGTH_STRING);
        break;
      case 'N':                 /* Mesh size 1 */
        N = strtoul(optarg, NULL, 10);
        break;
      case 'M':                 /* Mesh size 2 */
        M = strtoul(optarg, NULL, 10);
        break;
      case 'm':                 /* Friction coefficient */
        mu = atof(optarg);
        break;
      case 'n':                 /* Dimension */
        n = strtoul(optarg, NULL, 10);
        break;
      case 'l':                 /* LU */
        lu = 1;
        break;
      case 'q':                 /* QR */
        qr = 1;
        break;
      case 't':                 /* Tolerance */
        tol = atof(optarg);
        break;
      case 'e':                 /* Epsilon */
        epsilon = atof(optarg);
        break;
      case 's':                 /* Name input Fourier coefficients file */
        strncpy(input_fourier, optarg, MAX_LENGTH_STRING);
        break;
      case 'o':                 /* Name output file */
        strncpy(output, optarg, MAX_LENGTH_STRING);
        break;
      case 'p':                 /* Number of threads */
        #if defined (_OPENMP)
        omp_set_num_threads(atoi(optarg));
        #endif
        break;
      default:
        print_usage(argv[0]);
        return EXIT_FAILURE;
    }
  }
  
  if (optind < argc) {
    printf("non-option ARGV-elements: ");
    while (optind < argc)
      printf("%s ", argv[optind++]);
    printf("\n");
  }
  
  w = 2 * M_PI * w1 / w2;
  
  if (N == 0 || M == 0) {
    perror("Mesh size can not be zero");
    return EXIT_FAILURE;
  }
  
  double *X;
  if (flag != -1) {
    
    if ((X = pic_loadas_fourier_coef(n, N, output_fourier)) == NULL)
      return EXIT_FAILURE;
    
    if (surface(n, N, M, X, w1, w2, output_surface) == 0)
      fprintf(stderr, "Some error in paraview method\n");
    
    free(X);
    
    return EXIT_SUCCESS;
  } else {    
    if ((X = pic_loadas_fourier_coef(n, N, input_fourier)) == NULL)
      return EXIT_FAILURE;
  }
  
//   double *X;
//   if ((X = (double *) calloc(n*(2*N+1), sizeof(double))) == NULL) {
//     perror("Unsuccess allocated memory for X");
//     return EXIT_FAILURE;
//   }
//   
//   X[0] = M_PI;
// //   X = NULL;
  
  /* Creates a pic struct */
  if ((p = pic_create(n, N, w, X, NULL, NULL)) == NULL)
    return EXIT_FAILURE;
  
  /* Changes the function pointer that compute the values for 'f' */
  pic_set_function_pointer(p, f);
  
  /* Choices that method you want to use for solving the linear systems */
  if (lu != -1)
    pic_set_function_solve(p, SOLVE_WITH_LU);
  else if (qr != -1)
    pic_set_function_solve(p, SOLVE_WITH_QR);
  
  /* Compute the curve */
  if (pic_execute(p, tol) != NULL) {
    /* Save the results */
    pic_saveas_curve(p, output);
    pic_saveas_fourier_coef(p, output_fourier);
    pic_saveas_differential_indexs(p, output_matrix);
    
    /* Compute the surface */
    if (surface(p->n, p->N, M, p->X, w1, w2, output_surface) == 0)
      fprintf(stderr, "Some error in paraview method\n");
    
  } else {
    fprintf(stderr, "ERROR\n");
  }
  
  /* Free memory */
  pic_destroy(p);
  
  return EXIT_SUCCESS;
}