#include "pic.h"
#include <getopt.h>
#include <string.h>

#define MAX_LENGTH_STRING 64
#define DEFAULT_TOLERANCE 1e-12
#define DEFAULT_MESH_SIZE 36
#define DEFAULT_DELTA 1.1

double delta = DEFAULT_DELTA;

void f(size_t m, size_t n, double * const x, double * const f, double ** const df, double w, double theta) {
  double r, c, s;
  
  r = sqrt( x[0] * x[0] + x[1] * x[1] );
  c = cos(r); s = sin(r);
  
  f[0] = (x[0] * c - x[1] * s) * delta;
  f[1] = (x[0] * s + x[1] * c) / delta;
  
  if (df == NULL) return;
  
  df[0][0] = (c - (x[0] * s + x[1] * c) * x[0] / r) * delta;
  df[0][1] = -(s + (x[0] * s + x[1] * c) * x[1] / r) * delta;
  df[1][0] = (s + (x[0] * c - x[1] * s) * x[0] / r) / delta;
  df[1][1] = (c + (x[0] * c - x[1] * s) * x[1] / r) / delta;
}

void print_usage(char * const name) {
  printf("Usage: %s \n\t-h, --help\t\tHelp instruction\n\t-n, --dim value\t\tDimension\n\t-N, --mesh-size value\tMesh size\n\t-l, --lu\t\tSolve the linear system of equations with LU descomposition\n\t-q, --qr\t\tSolve the linear system of equations with QR descomposition\n\t-t, --tolerance value\tTolerance of the numeric method (default: %1.2e)\n\t-d, --delta value\tExtern auxiliar variable (default: %1.2e)\n\t-w value\t\tReal number (irrational)\n\t-p, --proc value\tNumber of threads that will be used\n\t-s, --input-fourier name\tInput file name with Fourier's coefficients\n\t-o, --output name\tOutput file name with the solution\n\t--output-fourier name\tOutput file name with Fourier's coefficients\n\t--output-matrix name\tOutput file name with the differential\n", name, DEFAULT_TOLERANCE, DEFAULT_DELTA);
}

int main(int argc, char **argv) {
  double w, tol;
  size_t n, N;
  short int lu = -1, qr = 1;
  
  struct pic *p;
  
  /* Default values */
  tol = DEFAULT_TOLERANCE;
  n = 2;
  N = DEFAULT_MESH_SIZE;
  w = M_SQRT2; //sqrt(2)
  char input_fourier[MAX_LENGTH_STRING]  = "test3_init_fourier.dat";
  char output[MAX_LENGTH_STRING]         = "data/test3.dat";
  char output_fourier[MAX_LENGTH_STRING] = "data/test3_coef_fourier.dat";
  char output_matrix[MAX_LENGTH_STRING]  = "data/test3_differential.dat";
  
  static struct option long_options[] = {
    {"help",                    no_argument,            0,      'h'},
    {"lu",                      no_argument,            0,      'l'},
    {"qr",                      no_argument,            0,      'q'},
    {"dim",                     required_argument,      0,      'n'},
    {"mesh-size",               required_argument,      0,      'N'},
    {"tolerance",               required_argument,      0,      't'},
    {"delta",                   required_argument,      0,      'd'},
    {"proc",                    required_argument,      0,      'p'},
    {"input-fourier",           required_argument,      0,      's'},
    {"output",                  required_argument,      0,      'o'},
    {"output-fourier",          required_argument,      0,      1},
    {"output-matrix",           required_argument,      0,      2},
    {NULL,                      0,                      NULL,   0}
  };
  int long_index = 0;
  
  /* Evaluate arguments */
  int c = 0;
  while ((c = getopt_long(argc, argv, "hN:n:lqt:w:d:s:o:p:", long_options, &long_index)) != EOF) {
    switch (c) {
      case 1:                   /* Name output Fourier coefficients file */
        strncpy(output_fourier, optarg, MAX_LENGTH_STRING);
        break;
      case 2:                   /* Name output matrix file */
        strncpy(output_matrix, optarg, MAX_LENGTH_STRING);
        break;
      case 'N':                 /* Mesh size */
        N = strtoul(optarg, NULL, 10);
        break;
      case 'n':                 /* Dimension */
        n = strtoul(optarg, NULL, 10);
        break;
      case 'l':                 /* LU */
        lu = 1;
        break;
      case 'q':                 /* QR */
        qr = 1;
        break;
      case 't':                 /* Tolerance */
        tol = atof(optarg);
        break;
      case 'w':                 /* Irrational number */
        w = atof(optarg);
        break;
      case 'd':                 /* Delta */
        delta = atof(optarg);
        break;
      case 's':                 /* Name input Fourier coefficients file */
        strncpy(input_fourier, optarg, MAX_LENGTH_STRING);
        break;
      case 'o':                 /* Name output file */
        strncpy(output, optarg, MAX_LENGTH_STRING);
        break;
      case 'p':                 /* Number of threads */
        #if defined (_OPENMP)
        omp_set_num_threads(atoi(optarg));
        #endif
        break;
      default:
        print_usage(argv[0]);
        return EXIT_FAILURE;
    }
  }
  
  if (optind < argc) {
    printf("non-option ARGV-elements: ");
    while (optind < argc)
      printf("%s ", argv[optind++]);
    printf("\n");
  }
  
  if (N == 0) {
    perror("Mesh size can not be zero");
    return EXIT_FAILURE;
  }
  
  double *X;
  if ((X = pic_loadas_fourier_coef(n, N, input_fourier)) == NULL)
    return EXIT_FAILURE;
  
  /* Changes the initial condition */
//   double *X;
//   if ((X = (double *) calloc(n*(2*N+1), sizeof(double))) == NULL) {
//     perror("Unsuccess allocated memory for X");
//     return EXIT_FAILURE;
//   }
//   
//   X[2] = w;
//   X[5] = w;
  
  /* Creates a pic struct */
  if ((p = pic_create(n, N, w, X, NULL, NULL)) == NULL)
    return EXIT_FAILURE;
  
  /* Changes the function pointer that compute the values for 'f' */
  pic_set_function_pointer(p, f);
  
  /* Choices that method you want to use for solving the linear systems */
  if (lu != -1)
    pic_set_function_solve(p, SOLVE_WITH_LU);
  else if (qr != -1)
    pic_set_function_solve(p, SOLVE_WITH_QR);
  
  /* Compute the curve */
  if (pic_execute(p, tol) != NULL) {
    /* Save the results */
    pic_saveas_curve(p, output);
    pic_saveas_fourier_coef(p, output_fourier);
    pic_saveas_differential_indexs(p, output_matrix);
  } else {
    fprintf(stderr, "ERROR\n");
  }
  
  /* Free memory */
  pic_destroy(p);
  
  return EXIT_SUCCESS;
}
