#include <stdlib.h>
#include <stdio.h>
#include <math.h>
#include <getopt.h>
#include <string.h>

#define MAX_LENGTH_STRING 64
#define DEFAULT_TOLERANCE 1e-12
#define DEFAULT_MESH_SIZE 8
#define DEFAULT_EPSILON 1e-2
#define DEFAULT_ALPHA 0.5

double epsilon = DEFAULT_EPSILON;
double alpha = DEFAULT_ALPHA;

void f(size_t m, size_t n, double * const x, double * const f, double ** const df, double w, double theta) {
  f[0] = cos(alpha) * x[0] - sin(alpha)*x[1] + sin(alpha) * x[0] * x[0] + epsilon * cos(theta);
  f[1] = sin(alpha) * x[0] + cos(alpha)*x[1] - cos(alpha) * x[0] * x[0] + epsilon * sin(theta);
  
  if (df == NULL) return;
  
  df[0][0] = cos(alpha) + 2 * sin(alpha) * x[0];
  df[0][1] = -sin(alpha);
  df[1][0] = sin(alpha) - 2 * cos(alpha) * x[0];
  df[1][1] = cos(alpha);
}

void print_usage(char * const name) {
  printf("Usage: %s \n\t-h, --help\t\tHelp instruction\n\t-n, --dim value\t\tDimension\n\t-N, --mesh-size value\tMesh size\n\t-l, --lu\t\tSolve the linear system of equations with LU descomposition\n\t-q, --qr\t\tSolve the linear system of equations with QR descomposition\n\t-t, --tolerance value\tTolerance of the numeric method (default: %1.2e)\n\t-e, --epsilon value\tExtern auxiliar variable (default: %1.2e)\n\t-a, --alpha value\tExtern auxiliar variable (default: %1.2e)\n\t-w value\t\tReal number (irrational)\n\t-p, --proc value\tNumber of threads that will be used\n\t-o, --output name\tOutput file name with the solution\n\t-s, --input-fourier name\tInput file name with Fourier's coefficients\n\t--output-fourier name\tOutput file name with Fourier's coefficients\n\t--output-matrix name\tOutput file name with the differential\n", name, DEFAULT_TOLERANCE, DEFAULT_EPSILON, DEFAULT_ALPHA);
}

int main(int argc, char **argv) {
  double w, tol;
  size_t n, N;
  short int lu = -1, qr = 1;
  
  struct pic *p;
  
  tol = DEFAULT_TOLERANCE;
  n = 2;
  N = DEFAULT_MESH_SIZE;
  w = M_PI * (sqrt(5)-1);
  char input_fourier[MAX_LENGTH_STRING]  = "test2_init_fourier.dat";
  char output[MAX_LENGTH_STRING]         = "data/test2.dat";
  char output_fourier[MAX_LENGTH_STRING] = "data/test2_coef_fourier.dat";
  char output_matrix[MAX_LENGTH_STRING]  = "data/test2_differential.dat";
  
  static struct option long_options[] = {
    {"help",                    no_argument,            0,      'h'},
    {"lu",                      no_argument,            0,      'l'},
    {"qr",                      no_argument,            0,      'q'},
    {"dim",                     required_argument,      0,      'n'},
    {"mesh-size",               required_argument,      0,      'N'},
    {"tolerance",               required_argument,      0,      't'},
    {"epsilon",                 required_argument,      0,      'e'},
    {"alpha",                   required_argument,      0,      'a'},
    {"proc",                    required_argument,      0,      'p'},
    {"input-fourier",           required_argument,      0,      's'},
    {"output",                  required_argument,      0,      'o'},
    {"output-fourier",          required_argument,      0,      1},
    {"output-matrix",           required_argument,      0,      2},
    {NULL,                      0,                      NULL,   0}
  };
  int long_index = 0;
  
  /* Evaluate arguments */
  int c = 0;
  while ((c = getopt_long(argc, argv, "hN:n:lqt:w:e:a:s:o:p:", long_options, &long_index)) != EOF) {
    switch (c) {
      case 1:                   /* Name output Fourier coefficients file */
        strncpy(output_fourier, optarg, MAX_LENGTH_STRING);
        break;
      case 2:                   /* Name output matrix file */
        strncpy(output_matrix, optarg, MAX_LENGTH_STRING);
        break;
      case 'N':                 /* Mesh size */
        N = strtoul(optarg, NULL, 10);
        break;
      case 'n':                 /* Dimension */
        n = strtoul(optarg, NULL, 10);
        break;
      case 'l':                 /* LU */
        lu = 1;
        break;
      case 'q':                 /* QR */
        qr = 1;
        break;
      case 't':                 /* Tolerance */
        tol = atof(optarg);
        break;
      case 'w':                 /* Irrational number */
        w = atof(optarg);
        break;
      case 'e':                 /* Epsilon */
        epsilon = atof(optarg);
        break;
      case 'a':                 /* Alpha */
        alpha = atof(optarg);
        break;
      case 's':                 /* Name input Fourier coefficients file */
        strncpy(input_fourier, optarg, MAX_LENGTH_STRING);
        break;
      case 'o':                 /* Name output file */
        strncpy(output, optarg, MAX_LENGTH_STRING);
        break;
      case 'p':                 /* Number of threads */
        #if defined (_OPENMP)
        omp_set_num_threads(atoi(optarg));
        #endif
        break;
      default:
        print_usage(argv[0]);
        return EXIT_FAILURE;
    }
  }
  
  if (optind < argc) {
    printf("non-option ARGV-elements: ");
    while (optind < argc)
      printf("%s ", argv[optind++]);
    printf("\n");
  }
  
  if (N == 0) {
    perror("Mesh size can not be zero");
    return EXIT_FAILURE;
  }
  
  double *X;
  if ((X = pic_loadas_fourier_coef(n, N, input_fourier)) == NULL)
    return EXIT_FAILURE;
  
  /* Creates a pic struct */
  if ((p = pic_create(n, N, w, X, NULL, NULL)) == NULL)
    return EXIT_FAILURE;
  
  /* Changes the function pointer that compute the values for 'f' */
  pic_set_function_pointer(p, f);
  
  /* Choices that method you want to use for solving the linear systems */
  if (lu != -1)
    pic_set_function_solve(p, get_solve_with_lu());
  else if (qr != -1)
    pic_set_function_solve(p, get_solve_with_qr());
  
  /* Compute the curve */
  if (pic_execute(p, tol) != NULL) {
    /* Save the results */
    pic_saveas_curve(p, output);
    pic_saveas_fourier_coef(p, output_fourier);
    pic_saveas_differential_indexs(p, output_matrix);
  } else {
    fprintf(stderr, "ERROR\n");
  }
  
  /* Free memory */
  pic_destroy(p);
  
  return EXIT_SUCCESS;
}
