#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <getopt.h>

#define MAX_LENGTH_STRING 64

/* The solution of the linear system equation will always be one's */
void init_easy_coef(size_t m, size_t n, double ** const A, double * const b) {
  typeof (m) i, j;
  typeof (*b) tmp;
  
  for (i = 0; i < m; i ++) {
    tmp = 0.;
    
    for (j = 0; j < n; j ++) 
      tmp += A[i][j];
    
    b[i] = tmp;
  }
}

void print_usage(char * const name) {
  printf("Usage: %s \n\t-h, --help\t\tHelp instruction\n\t-m, --rows value\tNumber of rows\n\t-n, --cols value\tNumber of columns\n\t-l, --lu\t\tSolve the linear system of equations with LU descomposition\n\t-q, --qr\t\tSolve the linear system of equations with QR descomposition\n\t--easy-solution\t\tThe independent coefficients are the sum of the row's matrix\n\t-p, --proc value\tNumber of threads that will be used\n\t-o, --output name\tOutput file name with the solution\n\t--output-matrix name\tOutput file name with the descomposition of the matrix linear system\n\t--input-matrix\t\tInput file name with the linear matrix\n\t--input-vector\t\tInput filename with the independent coefficients\n", name);
}

int main(int argc, char **argv) {
  int lu = -1, qr = -1, easy = -1;
  size_t m = 0, n = 10, i, j;
  double **A, *b;
  
  /* Default values */
  char input_matrix[MAX_LENGTH_STRING]  = "test_matrix.txt";
  char input_vector[MAX_LENGTH_STRING]  = "test_coef.txt"; 
  char output[MAX_LENGTH_STRING]        = "data/solution.dat";
  char output_matrix[MAX_LENGTH_STRING] = "data/output_matrix.dat";
  
  static struct option long_options[] = {
    {"help",                    no_argument,            0,      'h'},
    {"easy-solution",           no_argument,            0,      1},
    {"lu",                      no_argument,            0,      'l'},
    {"qr",                      no_argument,            0,      'q'},
    {"rows",                    required_argument,      0,      'm'},
    {"cols",                    required_argument,      0,      'n'},
    {"proc",                    required_argument,      0,      'p'},
    {"output",                  required_argument,      0,      'o'},
    {"output-matrix",           required_argument,      0,      2},
    {"input-matrix",            required_argument,      0,      3},
    {"input-vector",            required_argument,      0,      4},
    {NULL,                      0,                      NULL,   0}
  };
  int long_index = 0;
  
  /* Evaluate arguments */
  int c = 0;
  while ((c = getopt_long(argc, argv, "hlqm:n:o:p:", long_options, &long_index)) != EOF) {
    switch (c) {
      case 1:                 /* Easy solution */
        easy = 1;
        break;
      case 2:                 /* Name output matrix file */
        strncpy(output_matrix, optarg, MAX_LENGTH_STRING);
        break;
      case 3:                 /* Name input matrix file */
        strncpy(input_matrix, optarg, MAX_LENGTH_STRING);
        break;
      case 4:                 /* Name input vector file */
        strncpy(input_vector, optarg, MAX_LENGTH_STRING);        
        break;
      case 'l':                 /* LU */
        lu = 1;
        break;
      case 'q':                 /* QR */
        qr = 1;
        break;
      case 'm':                 /* Rows */
        m = strtoul(optarg, NULL, 10);
        break;
      case 'n':                 /* Columns */
        n = strtoul(optarg, NULL, 10);
        break;
      case 'o':                 /* Name output file */
        strncpy(output, optarg, MAX_LENGTH_STRING);
        break;
      case 'p':                 /* Number of threads */
        #if defined (_OPENMP)
        omp_set_num_threads(atoi(optarg));
        #endif
        break;
      default:
        print_usage(argv[0]);
        return EXIT_FAILURE;
    }
  }
  
  if (optind < argc) {
    printf("non-option ARGV-elements: ");
    while (optind < argc)
      printf("%s ", argv[optind++]);
    printf("\n");
  }
  
  if (m == 0) 
    m = n;
  
  /* Memory allocation */
  if ((b = (double *) calloc(n, sizeof(double))) == NULL) {
    fprintf(stderr, "Unsuccessful allocated memory for b\n");
    return EXIT_FAILURE;
  }
  
  if ((A = (double **) malloc(m * sizeof(double*))) == NULL) {
    free(b);
    fprintf(stderr, "Unsuccessful allocated memory for A\n");
    return EXIT_FAILURE;
  }
  
  for (i = 0; i < m; i ++) {
    if ((A[i] = (double *) calloc(n, sizeof(double))) == NULL) {
      fprintf(stderr, "Unsuccessful allocated memory the row %zu of A\n", i);
      
      for (j = 0; j < i; j ++)
        free(A[j]);
      free(A);
      free(b);
      
      return EXIT_FAILURE;
    }
  }
  
  /* Load linear system */
  loadas_matrix(A, input_matrix);
  
  if (easy == -1)
    loadas_vector(b, input_vector);
  else
    init_easy_coef(m, n, A, b);
  
  /* Solve linear system */
  if (qr != -1) {
    if (solve_qr(m, n, A, b, b, 0.) == NULL)
      fprintf(stderr, "Unsuccessful solve_qr\n");
  
  } else if (lu != -1) {
    if (solve_lu(m, n, A, b, b, 0.) == NULL)
      fprintf(stderr, "Unsuccessful solve_lu\n");
    
  } else {
    if (qrsis(m, n, A, b, 0.) == 0)
      fprintf(stderr, "Unsuccessful qrsis\n");
  }
  
  /* Save the results */
  saveas_vector(0, n, b, output);
  saveas_matrix(0, m, 0, n, A, output_matrix);
  
  /* Free memory */
  for (i = 0; i < m; i ++)
    free(A[i]);
  free(A);
  free(b);
  
  return EXIT_SUCCESS;
}