/*!
 * \file linear_solve.c
 * \brief Set of different function for solving linear systems 
 * 
 * \author Joan Gimeno
 * \date 22/11/2013 (start)
 * \version 1.0
 */

/*!
 * \defgroup solving_system Solving a linear of equations system
 * \brief Solving the system is find a vector \a x satisfying the linear equation Ax = b
 * 
 * \details The standard algorithm for solving a system of linear equations is based on Gaussian elimination with some modifications. Firstly, it is essential to avoid division by small numbers, which may lead to inaccurate results. This can be done by reordering the equations if necessary, a process known as pivoting. Secondly, the algorithm does not exactly do Gaussian elimination, but it computes the \ref LU of the matrix A. Other useful algorithm for solving a linear system is to compute \ref QR of the matrix A. This leads to the class of direct methods.\n
 * If the matrix A has some special structure, this can be exploited to obtain faster or more accurate algorithms. For instance, systems with a symmetric matrix or a symmetric positive definite matrix or a strictly diagonaly dominant matrix, ... Special methods exist also for matrices with many zero elements (so-called sparse matrices).\n
 * A completely different approach is often taken for very large systems, which would otherwise take too much time or memory. The idea is to start with an initial approximation to the solution (which does not have to be accurate at all), and to change this approximation in several steps to bring it closer to the true solution. Once the approximation is sufficiently accurate, this is taken to be the solution to the system. For instance, Jacobi method, Gauss-Seidel method, Successive over-relaxation (SOR) method, ... This leads to the class of iterative methods.\n
 * In this module, we implement some of these methods
 * 
 * @{ 
 */

#include "linear_solve.h"

/*!
 * \fn void forward_substitution_ones(size_t n, double ** const L, double * const x, double * const b)
 * \brief Forward substitution for lower triangular matrices (Lx = b) where the diagonal of L are 1's.
 * 
 * \param[in] n Dimension of matrix and vectors
 * \param[in] L Lower triangular matrix with 1's in its diagonal
 * \param[in] b Constant terms
 * \param[out] x Unknowns
 * 
 * \pre The pointers are not NULL
 * \post The vector x has the solution
 * \remarks The complexity is \f$ O(n^2) \f$
 */
void forward_substitution_ones(size_t n, double ** const L, double * const x, double * const b) {  
  typeof(n) i, j;
  typeof(**L) aux;
  
  x[0] = b[0];
  
  for (i = 1; i < n; i ++) {
    aux = 0.0;
    
    for (j = 0; j < i; j ++) 
      aux += L[i][j] * x[j];
    
    x[i] = b[i] - aux;
  }
}

/*!
 * \fn void * forward_substitution(size_t n, double ** const L, double * const x, double * const b, double tol)
 * \brief Forward substitution for lower triangular matrices (Lx = b)
 * 
 * \param[in] n Dimension of matrix and vectors
 * \param[in] L Lower triangular matrix
 * \param[in] b Constant terms
 * \param[out] x Unknowns
 * \param[in] tol Tolerance
 * 
 * \retval NULL If some diagonal's element is less than or equal to tolerance
 * \retval 1 Otherwise
 * 
 * \pre 
 *      - The pointers are not NULL
 *      - The tolerance is positive
 * \post The vector x has the solution
 * \remarks The complexity is \f$ O(n^2) \f$
 */
void * forward_substitution(size_t n, double ** const L, double * const x, double * const b, double tol) {
  typeof(n) i, j;
  typeof(**L) aux;
  
  for (i = 0; i < n; i ++) {
    if (fabs(L[i][i]) <= tol)
      return NULL;
    
    aux = 0.0;
    
    for (j = 0; j < i; j ++) 
      aux += L[i][j] * x[j];
    
    x[i] = (b[i] - aux) / L[i][i];
  }
  
  return (void *) 1;
}

/*!
 * \fn void * back_substitution(size_t n, double ** const U, double * const x, double * const b, double tol)
 * \brief Back substitution for upper triangular matrices (Ux = b)
 * 
 * \param[in] n Dimension of matrix and vectors
 * \param[in] U Upper triangular matrix
 * \param[in] b Constant terms
 * \param[out] x Unknowns
 * \param[in] tol Tolerance
 * 
 * \retval NULL If some diagonal's element is less than or equal to tolerance
 * \retval 1 Otherwise
 * 
 * \pre 
 *      - The pointers are not NULL
 *      - The tolerance is positive
 * \post The vector x has the solution
 * \note The complexity is \f$ O(n^2) \f$
 */
void * back_substitution(size_t n, double ** const U, double * const x, double * const b, double tol) {
  typeof(n) i, j;
  typeof(**U) aux;
  
  for (i = n-1; i < n; i --) { // NOTE Be careful is unsigned. Case extrem !
    if (fabs(U[i][i]) <= tol)
      return NULL;
    
    aux = 0.0;
    
    for (j = i+1; j < n; j ++) 
      aux += U[i][j] * x[j];
      
    x[i] = (b[i] - aux) / U[i][i];
  }
  
  return (void *) 1;
}

/*!
 * \fn void * solve_qr(size_t m, size_t n, double ** const A, double * const x, double * const b, double tol)
 * \brief QR solve linear system (QRx = b)
 * 
 * \param[in] n Dimension of matrix and vectors
 * \param[in,out] A Matrix's system
 * \param[in,out] b Constant terms
 * \param[out] x Unknowns
 *  
 * \retval NULL When QR factorization was unsuccessful or could not be allocated memory
 * \retval 1 Otherwise
 * 
 * \pre The dimension is greater than one
 * \post The x vector has the solution
 * \note 
 *      - The vector of constant terms will be modified
 *      - The unknowns pointer \a x and constant terms pointer \a b can be the same
 * \see qr
 */
void * solve_qr(size_t m, size_t n, double ** const A, double * const x, double * const b, double tol) {
  assert(n > 1);
  
  if (qr(m, n, A, A) == NULL) {
    #ifdef DEBUG
      printf("solve_qr: Unsuccessful QR decomposition\n");
    #endif
    return NULL;
  }
  
  premult_Q_vector(m, n, A, b);
  
  if (back_substitution(n, A, x, b, tol) == NULL) {
    #ifdef DEBUG
    printf("solve_qr: Unsuccessful back_substitution, some element is \n");
    #endif
    return NULL;
  }
  
  return (void *) 1;
}

/*!
 * \fn void * solve_lu(size_t m, size_t n, double ** const A, double * const x, double * const b, double tol)
 * \brief LU solve linear system (LUx = b)
 * 
 * \param[in] n Dimension of matrix and vectors
 * \param[in,out] A Matrix's system
 * \param[in] b Constant terms
 * \param[out] x Unknowns
 * \param[in] tol Tolerance
 * 
 * \retval NULL When LU factorization was unsuccessful or could not be allocated memory
 * \retval 1 Otherwise
 * 
 * \pre The dimension is greater than one
 * \post The x vector has the solution
 * \note 
 *      - The vector of constant terms will be modified
 *      - The unknowns pointer \a x and constant terms pointer \a b can be the same
 * \see lu
 */
void * solve_lu(size_t m, size_t n, double ** const A, double * const x, double * const b, double tol) {
  assert(n > 1);
  unsigned int flag = PARTIAL_PIVOTING_ROW;
  size_t *pr, *pc; /* Vector permutation */
  
  switch(flag) {
    case WHITOUT_PIVOTING:
      pr = NULL;
      pc = NULL;
      break;
    case PARTIAL_PIVOTING_ROW:
      if ((pr = (size_t *) malloc(m*sizeof(size_t))) == NULL) return NULL;
      pc = NULL;
      break;
    case PARTIAL_PIVOTING_COL:
      pr = NULL;
      if ((pc = (size_t *) malloc(n*sizeof(size_t))) == NULL) return NULL;
      break;
    case FULL_PIVOTING:
      if ((pr = (size_t *) malloc(m*sizeof(size_t))) == NULL) return NULL;
      if ((pc = (size_t *) malloc(n*sizeof(size_t))) == NULL) { free(pr); return NULL; }
      break;
    default:
      return NULL;
  }
  
  if (lu(m, n, A, A, pr, pc, tol) == NULL) {
    free(pc); free(pr);
    return NULL;
  }
  
  permutation_vector(m, pr, b, b);
  
  forward_substitution_ones(MIN(m, n), A, x, b);
  back_substitution(MIN(m, n), A, x, x, tol);
  
  permutation_vector(n, pc, x, x);
  
  free(pc); free(pr);
  return (void *) 1;
}

/*!
 * \fn void * solve(size_t m, size_t n, double ** const A, double * const x, double * const b, double tol, void * (*s)(size_t, size_t, double ** const, double * const, double * const, double))
 * \brief solve a linear system (Ax = b) with the pointer to a solve function
 * 
 * \param[in] n Dimension of matrix and vectors
 * \param[in,out] A Matrix's system
 * \param[in] b Constant terms
 * \param[out] x Unknowns
 * \param[in] tol Tolerance
 * \param[in] s Pointer to a function that solve the linear system
 * 
 * \return The value that is returned by the pointer to the solve function
 * 
 * \pre The dimension is greater than one
 * \post The x vector has the solution
 * \see \ref SOLVE_WITH_LU and \ref SOLVE_WITH_QR
 */
void * solve(size_t m, size_t n, double ** const A, double * const x, double * const b, double tol, 
             void * (*s)(size_t, size_t, double ** const, double * const, double * const, double)) {
  if (s == SOLVE_WITH_LU)
    return solve_lu(m, n, A, x, b, tol);
  
  if (s == SOLVE_WITH_QR)
    return solve_qr(m, n, A, x, b, tol);
  
  return NULL;
}

solve_ptr get_solve_with_lu() {
  return SOLVE_WITH_LU;
}

solve_ptr get_solve_with_qr() {
  return SOLVE_WITH_QR;
}
/*!
 * @}
 */